/* ------------------------------------------------------------------------
 *
 *        File: smg.c
 *     Project: Source Mapper.
 *     Created: April 7, 1992.
 * Description: This is the main source module of the project, which 
 *              defines the main() function.
 *
 * Copyright (C) 2000 Leif-Erik Larsen.
 * This file is part of the Source Mapper source package.
 * Source Mapper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, in version 2 as it comes in the
 * "COPYING" file of the XWorkplace main distribution.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ------------------------------------------------------------------------ */




#include "smg.h"




char progdir[MAXPATH];          /* Path to EXE-file of program        */
ErrorType Error;
int _iErrNo;                    /* Shadow of errno value */




struct
{
   int iCod;
   char* pMsg;
}
   ErrMsg[] =
{
   {  0, "Unknown error, or no error at all!" },
   {  1, "No matching file(s)!" },
   {  2, "Failed on open project file!" },
   {  3, "Failed on write to file!" },
   {  4, "Failed on create project file!" },
   {  5, "A rare error due to a bug in the program. Please write down the\n"
         "below information and a short description of what caused this\n"
         "error (if possible). Then contact the author to get help!" },
   {  6, "Can't fetch time stamp, or file not found!" },
   {  7, "Conflicting options: Can't link map without making .smo-files!" },
   {  8, "Unknown generate method!" },
   {  9, "Failed on create temporary file:" },
   { 10, "To many arguments from system! (Try to use an option file.)" },
   { 11, "Failed on open temporary file!" },
   { 12, "Failed on sort contents of file!" },
   { 13, "Seek error!" },
   { 14, "Out of memory! (Try to split source into smaller modules.)" },
   { 15, "Failed on read file!" },
   { 16, "Failed on open file!" },
   { 17, "Unknown or illegal argument of option! (Using default)" },
   { 18, "Only one sourcefile can be specified if some has wildcard(s)!" },
   { 19, "Unknown option or switch!" },
   { 20, "To long line in sourcefile! Max length of a line is" },
   { 21, "To many {-block-} levels in sourcefile (max is 15)" },
   { 22, "Failed on create final map file" },
   { 23, "Failed on open sourcefile" },
   { 24, "Failed on open options file" },
   { 25, "Illegal level of block. Probably due to a syntax error." },
   { 26, "Failed on copy filebytes to" },
   { 27, "Function name is to long. Max length is" },
   { 28, "Identifier name is to long. Max length is" },
};




char *ptbl[PT_MAXNR] =  /* Parametertabell (Parameter TaBLe) */
{
   /* ______ Totalinfo: ______ */
   "iti",    /* PT_iti: Inkluder */
   "clt",    /* PT_ctl: Antall linjer */
   "cel",    /* PT_cel: Antall tomme linjer */
   "coc",    /* PT_coc: Antall linjer med bare komentar */
   "clc",    /* PT_clc: Antall linjer med kode */
   "cfu",    /* PT_cfu: Antall funksjoner */
   "cco",    /* PT_cco: Antall komentarblokker */

   /* ______ Funksjonsregister: ______ */
   "ifr",    /* PT_ifr: Inkluder */
   "fso",    /* PT_fso: Sorteringsmetode (1=Navn, 2=Linjenummer) */

   /* ______ Liste over funksjonskall: ______ */
   "ilf",    /* PT_ilf: Inkluder */
   "slf",    /* PT_slf: Selektiv liste */
   "fln",    /* PT_fln: Stigende linjenummer-referanser */

   /* ______ Liste over identifikatorer: ______ */
   "iil",    /* PT_iil: Inkluder */
   "sli",    /* PT_sli: Selektiv liste */
   "iln",    /* PT_iln: Stigende linjenummer-referanser */

   /* ______ Kall-sekvenser: ______ */
   "csi",    /* PT_csi: Include */
   "cie",    /* PT_cie: Include external functions */
   "csg",    /* PT_csg: Use graphicals characters */
   "csf",    /* PT_csf: Format (1=List, 2=Flowchart) */
   "cwi",    /* PT_cwi: Width of indents */
   "cir",    /* PT_cir: Include indexregister */
   "cdt",    /* PT_cdt: Default top level function */

   /* ______ Linjeinformasjon: ______ */
   "ili",    /* PT_ili: Inkluder */
   "lll",    /* PT_lll: Lokale linjenummer */
   "rbi",    /* PT_rbi: Blokkniv */
   "lcl",    /* PT_lcl: Anngi rene komentarlinjer */

   /* ______ Informasjon om funksjoner: ______ */
   "lfi",    /* PT_lfi: Inkluder funksjonsinfo */
   "LLF",    /* PT_LLF: Antall linjer i funksjon */
   "LEF",    /* PT_LEF: Antall tomme linjer i funksjon */
   "LCF",    /* PT_LCF: Antall komentarlinjer i funksjon */
   "LCO",    /* PT_LCO: Antall kodelinjer i funksjon */
   "LCC",    /* PT_LCC: Antall komentarblokker i funksjon */
   "LLC",    /* PT_LLC: Liste over kallere av funksjonen */

   /* ______ Diverse: ______ */
   "NPs",    /* PT_NPs: Streng for sideskift */
   "NPS",    /* PT_NPS: Sideskift ved streng for sideskift */
   "NFU",    /* PT_NFU: Sideskift ved funksjonstart */
   "NFI",    /* PT_NFI: Sideskift mellom kart seksjoner */
   "cse",    /* PT_cse: Kolonneseparator (1=Tab, 2=Mellomrom) */
   "dtu",    /* PT_dtu: Slett mellomfiler (.smg-filer) etter bruk */
   "gcb",    /* PT_gcb: Generer komentarblokker */
   "tab",    /* PT_tab: Tabulatorbredde */
   "dis",    /* PT_dis: Display information on screen during processing */

   /* ______ Generer: ______ */
   "gen",    /* PT_gen: Generer (1=Alt, 2=Mellom, 3=Alt) */
   "met",    /* PT_met: Metode (1=Link, 2=Make, 3=Build) */
   "pof",    /* PT_pof: Katalog til mellomfiler */
   "pmf",    /* PT_pmf: Katalog til kartfiler */
   "ptf",    /* PT_ptf: Katalog til midlertidige filer */
   "upf",    /* PT_upf: Bruk prosjektfil */

   /* ______ Brytere som bare utfres direkte, uten  lagres i valg-fil: */
   "uof",    /* PT_uof: Bruk valgfil */
   "wof",    /* PT_wof: Lagre valgfil */
   "res",    /* PT_res: Reset valg. Valg settes tilbake til standard */
   "hlp",    /* PT_hlp: Gir bruker hjelp om oppgitt bryter */
};




char *defoptcom[PT_MAXNR] = /* DEFault OPTion COMments */
{
   /* ______ Totalinfo: ______ */
   "Include header with total information",                     /* PT_iti */
   "Count total number of lines in source",                     /* PT_ctl */
   "Count number of empty lines in source",                     /* PT_cel */
   "Count number of comment lines in source",                   /* PT_coc */
   "Count number of lines containing code in source",           /* PT_clc */
   "Count number of functions in source",                       /* PT_cfu */
   "Count number of comment blocks in source",                  /* PT_cco */

   /* ______ Funksjonsregister: ______ */
   "Include register of functions in source",                   /* PT_ifr */
   "Sorting method of functions register (1=Name, 2=Linr)",     /* PT_fso */

   /* ______ Liste over funksjonskall: ______ */
   "Include list of function calls in source",                  /* PT_ilf */
   "Selective list of functions in list of functioncalls",      /* PT_slf */
   "Rising line numbers references in list of function calls",  /* PT_fln */

   /* ______ Liste over identifikatorer: ______ */
   "Include list of identifiers in source",                     /* PT_iil */
   "Selective list of identifiers in list of identifiers",      /* PT_sli */
   "Rising line number references in list of identifiers",      /* PT_iln */

   /* ______ Kall-sekvenser: ______ */
   "Include list of calling sequences in source",               /* PT_csi */
   "Include external functions in list of calling sequences",   /* PT_cie */
   "Use graphical characters in list of calling sequences",     /* PT_csg */
   "Format of list of calling sequences (1=List, 2=Flowchart)", /* PT_csf */
   "Width of indents in list of calling sequences",             /* PT_cwi */
   "Include index register of calling sequences",               /* PT_cir */
   "Default top level function in list of calling sequences",   /* PT_cdt */

   /* ______ Linjeinformasjon: ______ */
   "Include source lines information",                          /* PT_ili */
   "Report local linenumber of each source line",               /* PT_lll */
   "Report level of block indention",                           /* PT_rbi */
   "Report each comment line in source",                        /* PT_lcl */
   "Include information about each function in source",         /* PT_lfi */
   "Report number of lines in each function in source",         /* PT_LLF */
   "Report number of empty lines in each function in source",   /* PT_LEF */
   "Report number of comments in each function in source",      /* PT_LCF */
   "Report number of code lines in each function in source",    /* PT_LCO */
   "Report number of comment lines in each function in source", /* PT_LCC */
   "Report list of callers of each function in source",         /* PT_LLC */

   /* ______ Diverse: ______ */
   "Source string to reqognice as new-page-string",             /* PT_NPs */
   "Convert new-page-string to a new-page char",                /* PT_NPS */
   "Insert new-page char before each function",                 /* PT_NFU */
   "Insert new-page char between each map section",             /* PT_NFI */
   "Column separator (1=space, 2=tab)",                         /* PT_cse */
   "Delete temporary .smo-files after use",                     /* PT_dtu */
   "Put all added information within C-style comment blocks",   /* PT_gcb */
   "Width of tabulator",                                        /* PT_tab */
   "Disp. info on screen (0=Nothing,1=Few,2=Normal,3=Detailed)",/* PT_dis */

   /* ______ Generer: ______ */
   "What to generate (1=All, 2=Objects, 3=Map)",                /* PT_gen */
   "Method of generator (1=Link, 2=Make, 3=Build)",             /* PT_met */
   "Path to object files (.smo-files)",                         /* PT_pof */
   "Path to map files (.sm-files)",                             /* PT_pmf */
   "Path to temporary files (.tmp-files)",                      /* PT_ptf */
   "Use project file",                                          /* PT_upf */

   /* ______ Brytere som bare utfres direkte, uten  lagres i valg-fil: */
   "Read and use options from specified text file",             /* PT_uof */
   "Write current options settings to specified text file",     /* PT_wof */
   "Reset all options back to default",                         /* PT_res */
   "User help",                                                 /* PT_hlp */
};




char *optblocks[] =
{
   "Header with total information about source",           /* OB_TOTINFO */
   "Register of functions in source",                      /* OB_FUNCREG */
   "List of function calls",                               /* OB_FUNCALL */
   "List of identifiers",                                  /* OB_IDLIST */
   "List of calling sequences",                            /* OB_CALSEQ */
   "Source lines information",                             /* OB_LINEINF */
   "Information about each function in source",            /* OB_FUNCINFO */
   "Various options",                                      /* OB_VARIOUS */
   "How and what to generate",                             /* OB_GENERATE */
};




char *fnames[] =
{
   ".c",           /* CSOURCE_EXT */
   ".tmp",         /* SMGTEMP_EXT */
   ".opt",         /* SMGOPTI_EXT */
   ".prj",         /* SMGPRJ_EXT */
   ".smo",         /* SMGOBJC_EXT */
   ".sm",          /* SMGMAPF_EXT */
   ".bak",         /* SMGBACK_EXT */
   "default",      /* DEFOPT_FNAME */
};




char _StrErrWarning[]         = "Warning: ";
char _StrErrUnknownOption[]   = "Unknown option: %s\n";
char _StrMSGCOPYRIGHT[]       = "Source Mapper ver. "VERSION" for "PLATFORM". Copyright (C) Leif-Erik Larsen, 2000.";
char _StrMSGTEMPFILEID[]      = "Source Mapper ver. "VERSION" for "PLATFORM". Objectfile.\n";
char _StrMSGPRJFILEID[]       = "Source Mapper ver. "VERSION" for "PLATFORM". Projectfile.\n";
char _StrMSGUSAGE[]           = "Usage: sm [Option1] [Option2] ... Source1 [Source2] ...\nFor help on availible options, type 'sm hlp[]'\n";
char _StrMSGINTERPRETLIN[]    = "Scanning lines in source";
char _StrMSGSORTFUNCREG[]     = "Sorting functions";
char _StrMSGSORTIDLST[]       = "Sorting identifiers";
char _StrMSGSORTFCALLST[]     = "Sorting function calls";
char _StrMSGINCLFUNDATA[]     = "Collecting functions";
char _StrMSGINCLLINEINFO[]    = "Collecting data about lines";
char _StrMSGINCLIDDATA[]      = "Collecting identifiers";
char _StrMSGINCLFCALDATA[]    = "Collecting function calls";
char _StrMSGMAKEDIFIDF[]      = "Finding identifiers";
char _StrMSGMAKEDIFFCALF[]    = "Finding function calls";
char _StrMSGMAKINDXFCAL[]     = "Indexing function calls";
char _StrMSGMAKINDXIDLST[]    = "Indexing identifiers";
char _StrMSGTOTINF_SNAME[]    = "Sourcefile";
char _StrMSGTOTINF_LINTOT[]   = "Total number of lines";
char _StrMSGTOTINF_EMPLIN[]   = "Empty lines";
char _StrMSGTOTINF_COMLIN[]   = "Commentlines";
char _StrMSGTOTINF_CODLIN[]   = "Lines with code";
char _StrMSGTOTINF_FUNTOT[]   = "Functions";
char _StrMSGTOTINF_COMENT[]   = "Blocks of comment";
char _StrMSGTOTALYSTR[]       = "Totaly";
char _StrMSGGENERATESOBJ[]    = "Generating object file(s)";
char _StrMSGGENERATESMAP[]    = "Generating map";
char _StrMSGINCLTOTINFO[]     = "Making total information";
char _StrMSGGLOBLINSTART[]    = "Glob.lin.";
char _StrMSGINCLFUNCREG[]     = "Making register of functions";
char _StrMSGFUNCREGSTR[]      = "Index of functions";
char _StrMSGNOFUNCTIONS[]     = "No functions";
char _StrMSGTOTFUNCTIONS[]    = "Total number of functions";
char _StrMSGINCLFUNCALLST[]   = "Making list of function calls";
char _StrMSGFUNCALLSTSTR[]    = "List of function calls";
char _StrMSGSELFUNCALLSTSTR[] = "Selective list of function calls";
char _StrMSGTOTFUNCCALS[]     = "Total number of blocks with function calls in the list";
char _StrMSGIDLSTSTR[]        = "List of identifiers";
char _StrMSGSELIDLSTSTR[]     = "Selective list of identifiers";
char _StrMSGTOTIDS[]          = "Total number of identifiers in the list";
char _StrMSGINCLIDLST[]       = "Including list of identifiers";
char _StrMSGINCLINEINFO[]     = "Source code and line information";
char _StrMSGSOURCECODEINF[]   = "Source with lineinfo";
char _StrMSGONLYCOMENT[]      = "Comment";
char _StrMSGLISTOFCALLERS[]   = "List of callers of";
char _StrMSGGENERALFUNCINF[]  = "General info about";
char _StrMSGFUNCINF_LINTOT[]  = "Total number of lines";
char _StrMSGFUNCINF_LINEMP[]  = "Empty lines";
char _StrMSGFUNCINF_LINCOM[]  = "Commentlines";
char _StrMSGFUNCINF_LINCOD[]  = "Lines with code";
char _StrMSGFUNCINF_CBLOCS[]  = "Blocks of comment";
char _StrMSGNOBODYCALLFUNC[]  = "No caller";
char _StrMSGSEEKTPLEVELF[]    = "Searching for top level function(s)";
char _StrMSGNOTOPLEVELFUNC[]  = "Warning! No top level function. Tries to use default!";
char _StrMSGFLOWCLSTSTR[]     = "List of calling sequences";
char _StrMSGINCLFLOWC[]       = "Making list of calling sequences";
char _StrMSGEXTERNAL[]        = "External";
char _StrMSGFLOWGENSTRIP[]    = "Making stripped references of all calling references";
char _StrMSGSORTFLOWIDXREG[]  = "Sorting index of calling secuences";
char _StrMSGINCLFLOWIDXREG[]  = "Making index of calling sequences";
char _StrMSGIDXREGFLOW[]      = "Index to list of calling sequences";




void ErrorShow ( void )
{
   switch (Error.iErr)
   {
      case   0: DoError ("%s",          ErrMsg[  0].pMsg); break;
      case   1: DoError ("%s (%s)",     ErrMsg[  1].pMsg, Error.cFName); break;
      case   2: DoError ("%s (%s)",     ErrMsg[  2].pMsg, Error.cFName); break;
      case   3: DoError ("%s (%s)",     ErrMsg[  3].pMsg, Error.cFName); break;
      case   4: DoError ("%s (%s)",     ErrMsg[  4].pMsg, Error.cFName); break;
      case   5: DoError ("%s",          ErrMsg[  5].pMsg); break;
      case   6: DoError ("%s (%s)",     ErrMsg[  6].pMsg, Error.cFName); break;
      case   7: DoError ("%s",          ErrMsg[  7].pMsg); break;
      case   8: DoError ("%s",          ErrMsg[  8].pMsg); break;
      case   9: DoError ("%s %s",       ErrMsg[  9].pMsg, Error.cFName); break;
      case  10: DoError ("%s",          ErrMsg[ 10].pMsg); break;
      case  11: DoError ("%s (%s)",     ErrMsg[ 11].pMsg, Error.cFName); break;
      case  12: DoError ("%s (%s)",     ErrMsg[ 12].pMsg, Error.cFName); break;
      case  13: DoError ("%s (%s)",     ErrMsg[ 13].pMsg, Error.cFName); break;
      case  14: DoError ("%s",          ErrMsg[ 14].pMsg); break;
      case  15: DoError ("%s (%s)",     ErrMsg[ 15].pMsg, Error.cFName); break;
      case  16: DoError ("%s (%s)",     ErrMsg[ 16].pMsg, Error.cFName); break;
      case  17: DoError ("%s (%s)",     ErrMsg[ 17].pMsg, Error.cFName); break;
      case  18: DoError ("%s",          ErrMsg[ 18].pMsg); break;
      case  19: DoError ("%s (%s)",     ErrMsg[ 19].pMsg, Error.cFName); break;
      case  20: DoError ("%s %d. (%s)", ErrMsg[ 20].pMsg, MAXINLIN, Error.cFName); break;
      case  21: DoError ("%s (%s)",     ErrMsg[ 21].pMsg, Error.cFName); break;
      case  22: DoError ("%s (%s)",     ErrMsg[ 22].pMsg, Error.cFName); break;
      case  23: DoError ("%s (%s)",     ErrMsg[ 23].pMsg, Error.cFName); break;
      case  24: DoError ("%s (%s)",     ErrMsg[ 24].pMsg, Error.cFName); break;
      case  25: DoError ("%s",          ErrMsg[ 25].pMsg, Error.cFName); break;
      case  26: DoError ("%s %s",       ErrMsg[ 26].pMsg, Error.cFName); break;
      case  27: DoError ("%s %d.",      ErrMsg[ 27].pMsg, MAXFUNCN); break;
      case  28: DoError ("%s %d.",      ErrMsg[ 28].pMsg, MAXFUNCN); break;
   }
} /* ErrorShow (); */




int ErrorClear ( void )
/* Clear the error code used by our program, making it possible to record
   another error status later.
   Return: Previous error code. */
{
   int iRet = Error.iErr;

   _iErrNo = 0;
   memset (&Error, 0, sizeof (Error));

   return iRet;
} /* ErrorClear (); */




int ErrorSet ( int iErrCode, const char *pFName,
                             const char *pSFileName, int iSFileLNr )
/* Record specified error.
   iErrCode   = Actual errro code (see static structure ErrMsg).
   pFName     = Name of the file that caused the error, or NULL if no such file.
   pSFileName = Name of the source file that called this function (__FILE__).
   pSFileLNr  = Line number of source file where this function was
                called (__LINE__).
   Return: Previous error code. */
{
   int iRet = Error.iErr;

   if (Error.iErr)
      return iRet;                     /* Error has already been recorded */

   Error.iErr       = iErrCode;
   Error.errno      = _iErrNo;
   Error.pSFileName = (char*) pSFileName;
   Error.iSFileLNr  = iSFileLNr;

   if (pFName)
      strcpy (Error.cFName, pFName);

   if (iErrCode == 7  ||     /* Conflictiong options ... */
       iErrCode == 8  ||     /* Unknown generate method! */
       iErrCode == 9  ||     /* Illegal name of SM executable file! */
       iErrCode == 10 ||     /* To many arguments from system! */
       iErrCode == 17 ||     /* Unknown or illegal argument of option! */
       iErrCode == 18 ||     /* Only one sourcefile can bee specified ... */
       iErrCode == 19 ||     /* Unknown option or switch! */
       iErrCode == 20 ||     /* To long line in source! */
       iErrCode == 21 ||     /* To many block levels in source */
       iErrCode == 25 ||     /* Illegal level of block */
       iErrCode == 27 ||     /* Function name is to long */
       iErrCode == 28)       /* Identifier name is to long */
   {
      /* The specified error is of a type that the system doesn't know. */
      Error.errno = 0;
   }

   return iRet;
} /* ErrorSet (); */




void DoError ( const char *format, ... )
/* Opprettet: Onsdag 21. april 1993.
   Parameter: "format"  Formatstreng for feilmelding.
              "..."     Feilmelding til bruker.
   Retur    :
   Beskriv  : Viser oppgitt advarsel p sjkerm. Inkluderer et linjeskift
              bde forran og etter den oppgitte feilmeldingen.
*/
{
   va_list argp;                       /* Peker til aktiv parameter */

   printf ("\n"
           "Error: ");

   va_start (argp, format);            /* Initialiserer "argpek" */
   vprintf  (format, argp);            /* Skriv oppgitte strenger, til bruker*/
   va_end   (argp);                    /* Avslutter alle parameterne */

   printf ("\n"
           "\n"
           "    Error #: %d\n"
           "  SM module: %s\n"
           "  SM line #: %d\n"
           "Msg from OS: %d = %s\n"
           "\n",

           Error.iErr,
           Error.pSFileName ? Error.pSFileName : "N/A",
           Error.iSFileLNr,
           Error.errno,
           Error.errno ? strerror (Error.errno) : "None");

   return;
} /* DoError (); */




void DoWarning ( const char *format, ... )
/*  Function: Show specified warning message on display.
   Opprettet: Lrdag 21. oktober 1995.
   Parameter: "format"  Formatstreng for feilmelding.
              "..."     Feilmelding til bruker.
*/
{
   va_list argp;            /* Peker til aktiv parameter */
   printf (_StrErrWarning);
   va_start (argp, format); /* Initialiserer "argpek" */
   vprintf (format, argp);  /* Skriv oppgitte strenger, til bruker*/
   va_end (argp);           /* Avslutter alle parameterne */
} /* DoWarning (); */




void SMHelpCmd ( const char *pStrHelp )
{
   if (strcmp (pStrHelp, "") == 0)
   {
      printf ("Here is a list of some frequently used options:\n"
              "\n"
              "%s[+/-] %s\n"
              "%s[+/-] %s\n"
              "%s[+/-] %s\n"
              "%s[+/-] %s\n"
              "%s[+/-] %s\n"
              "%s[+/-] %s\n"
              "%s[+/-] %s\n"
              "%s[+/-] %s\n"
              "%s[...] %s\n"
              "%s[...] %s\n"
              "\n"
              "( --- Note: All options are case significant --- )\n"
              "\n"
              "You can type 'sm hlp[more]' to get more help.\n",
              ptbl[PT_iti], defoptcom[PT_iti],
              ptbl[PT_ifr], defoptcom[PT_ifr],
              ptbl[PT_ilf], defoptcom[PT_ilf],
              ptbl[PT_iil], defoptcom[PT_iil],
              ptbl[PT_csi], defoptcom[PT_csi],
              ptbl[PT_cir], defoptcom[PT_cir],
              ptbl[PT_ili], defoptcom[PT_ili],
              ptbl[PT_lfi], defoptcom[PT_lfi],
              ptbl[PT_uof], defoptcom[PT_uof],
              ptbl[PT_wof], defoptcom[PT_wof]);
   }

   else
   if (strcmp (pStrHelp, "more") == 0)
   {
      printf ("Look at the contents of the file '%s%s' to get a short\n"
              "description of every documented option. If you don't have any such file in\n"
              "your directory, type 'sm wof[]' to let SM create it for you. This option\n"
              "file will be loaded automatically each time SM is executed. Therefore, you\n"
              "can edit the contents of this file to change the default settings of SM.\n"
              "\n"
              "Some options are not listed in the file '%s%s'. Those are the\n"
              "options which are executed directly when you specify them, without being\n"
              "stored as real options in the memory of SM. Here you have a list of them:\n"
              "\n"
              "%s[...] %s\n"
              "%s[...] %s\n"
              "%s[]    %s\n"
              "%s[]    %s\n",
              fnames[DEFOPT_FNAME], fnames[SMGOPTI_EXT],
              fnames[DEFOPT_FNAME], fnames[SMGOPTI_EXT],
              ptbl[PT_uof], defoptcom[PT_uof],
              ptbl[PT_wof], defoptcom[PT_wof],
              ptbl[PT_res], defoptcom[PT_res],
              ptbl[PT_hlp], defoptcom[PT_hlp]);
   }

   else
   {
      printf ("Unknown help string: %s\n", pStrHelp);
   }
} /* SMHelpCmd (); */




void ClrLin ( void )
/* Opprettet: Sndag 6. juni 1992.
   Beskriv  : Tmmer hele linjen hvor cursor befinner seg.
              Cursoren plasseres inntill venstre bildekant.
*/
{
   Display (D_HEIGHEST, "");
} /* ClrLin (); */




void ClrEOL ( void )
/* Created: Tirsdag 29. september 1992.
   Beskriv: Tmmer resten av linjen hvor cursor befinner seg.
            Cursoren flyttes ikke.
*/
{
   #if __IBMC__ || __IBMCPP__

   /* I couldn't find any clreol() in the Visual Age C++ library, so I had
      to hack this little temporary uggly code... */

   static int iWasHere = FALSE;
   static int iNrOfColumns = 0;
   int iC1;
   char cStr[256];
   USHORT Row;
   USHORT Col;
   VIOMODEINFO vinfo;

   if (!iWasHere)
   {
      iWasHere = TRUE;
      vinfo.cb = sizeof (vinfo);
      VioGetMode (&vinfo, (HVIO) 0);
      iNrOfColumns = vinfo.col;
      if (iNrOfColumns >= sizeof (cStr))
         iNrOfColumns = sizeof (cStr) - 1;
      iNrOfColumns -= 1; /* Don't print to rightmost column */
   }

   VioGetCurPos (&Row, &Col, (HVIO) 0);
   for (iC1 = 0; iC1 < iNrOfColumns - Col; cStr[iC1++] = ' ');
   cStr[iC1] = '\0';
   printf (cStr);
   VioSetCurPos (Row, Col, (HVIO) 0);

   #else
   clreol ();
   #endif
} /* ClrEOL (); */




int Display ( DISP_PRIORITY iPriority, const char *format, ... )
/*
    Function: Display a string on current line on display.
        Date: April, 26. 1993. 13:05. By LEL.
   Interface: iPriority = Priority level of message to display. This value
                          is used to filter how important the message is, and
                          compare that to the optioned message level
                          specified by the user.
              format    = Format string, as in 'printf ()' function.
              ...       = List og arguments.
     Returns: Number of written characters.
*/
{
   int count;
   va_list argptr;

   switch (o_various.iShowInfo)
   {
      case 0:  /* Nothing */
               return 0;

      case 1:  /* Few */
               if (iPriority < D_HEIGHEST)
                  return 0;

      case 2:  /* Normal */
               if (iPriority < D_HEIGH)
                  return 0;

      case 3:  /* Detailed */
               if (iPriority < D_LOW)
                  return 0;
   }

   printf ("\r"); /* Position cursor on first column */

   va_start (argptr, format);
   count = vprintf (format, argptr);
   va_end (argptr);

   ClrEOL ();

   return (count);
} /* Display (); */




void *Alloc ( long size )
/*
    Function: Allocates memory from heap.
        Date: March, 17. 1993. LEL.
   Interface: size = Number of bytes to allocate.
     Returns: A pointer to the allocated memory-block.
              A NULL pointer if out of memory.
*/
{
   if (size <= 0)
   {
      ERRORSET (5, NULL); /* A rare error that should never happen! */
      return NULL;
   }

   #if defined (__GNUC__) || defined (__OS2__)
   return (calloc (size, sizeof (char)));
   #else
   return (farcalloc (size, sizeof (char)));
   #endif
} /* Alloc (); */




void Free ( void *buff )
/* Function: Gives allocated memory-block back to heap.
       Date: March, 17. 1993. By LEL. */
{
   if (buff == NULL)
      return;

   #if defined (__GNUC__) || defined (__OS2__)
   free (buff);
   #else
   farfree (buff);
   #endif
} /* Free (); */




long MemLeft ( void )
/* Created: Fredag 16. october 1992.
   Return : Size of the largest free memory pool in heap. */
{
   #if defined (__GNUC__) || defined (__OS2__) || defined (__WIN32__)
   long ret = 0x7FFFFFFF;        /* 32 bits ==> "Unlimited" memory heap */
   #else
   long ret = farcoreleft ();
   #endif
   return (ret);
} /* MemLeft (); */




char *Adress ( char *buff, long index, int esize )
/*
    Function: Calculate the physical address of indexed element in buffer.
        Date: April, 29. 1993. 03:40. By LEL.
   Interface: buff  = Address of buffer witch is to be indexed.
              index = Index to element in buffer (0..).
              esize = Size of each element in buffer. Size must be
                      specified with number of bytes.
     Returns: Address to indexed element in buffer.
     Comment: This function is included for use when need to access
              data in buffer which is larger than 64K.
*/
{
   char huge *ret;

   ret  = buff;
   ret += index * esize;

   return ((char far *) ret);
} /* Adress (); */




int MakeMapFiles ( int job, const char *path )
/*
    Function: Make map files and eventually objectfiles of files
              specified by filename with wildcharacters.
        Date: Februar, 23. 1992. By LEL.
   Interface: "job"  PRJMAKE, PRJBUILD or PRJLINK.
              "path" Path and name of files.
     Returns: E/O.
*/
{
   int       countf = 0;               /* Teller filer som passer jokerfil */
   char      wildchrf [MAXPATH + 1];   /* Navn til joker-fil */
   char      sfname [MAXPATH + 1];     /* Navn m/katalog til kildefil */
   prjfeTYPE fdata;                    /* Mellomlager for kildefildata */

   #if __IBMC__ || __IBMCPP__
   APIRET rc;
   HDIR FindHandle = HDIR_CREATE;
   ULONG ulFindCount = 1;              /* Don't read more than one filename at once */
   FILEFINDBUF3 FFBuff;                /* File Info including size of EA's */
   #define isDONE (rc != NO_ERROR)
   #define thisFILENAME (FFBuff.achName)
   #define isLEGALFILE (!(FFBuff.attrFile & FILE_DIRECTORY))
   #else
   int done;
   struct ffblk _ffblk;                /* DOS file control block structure */
   #define isDONE (done)
   #define thisFILENAME (_ffblk.ff_name)
   #define isLEGALFILE (!((_ffblk.ff_attrib & FA_LABEL) || (_ffblk.ff_attrib & FA_DIREC)))
   #endif

   strcpy (wildchrf, path);            /* Lagre navn til joker-fil */
   strcpy (sfname, path);              /* Lagre katalog til kildefil(er) */

   #if __IBMC__ || __IBMCPP__
   rc = DosFindFirst ("*.*", &FindHandle,
                      FILE_ARCHIVED | FILE_SYSTEM | FILE_READONLY | FILE_HIDDEN,
                     (PVOID) &FFBuff,
                      sizeof (FFBuff), &ulFindCount, FIL_STANDARD);
   #else
   done = findfirst (path, &_ffblk, 0);
   #endif
   while (!isDONE)                     /* Loop through findnext until done */
   {
      if (isLEGALFILE)                 /* Ignore directories */
      {
         countf += 1;                  /* Teller antall matchende filnavn */

         ResetPrjStat ();              /* Resett prosjekt-status */
         ResetPrjFETYPE (&fdata);      /* Resett prosjekt-data */

         CutFName (sfname);            /* Slett filnavn del av katalog */
         Slash (sfname);
         strcat (sfname, thisFILENAME);/* Konstruer fullt navn til kildefil */

         strcpy (fdata.name, sfname);  /* Lagre kildefilnavn til prosjekt */

         strcpy (fdata.objn, sfname);  /* Lagre navn til objektfilen ogs */
         CutFExt (fdata.objn);
         strcat (fdata.objn, fnames[SMGOBJC_EXT]);

         /* Lagre informasjon om denne kildemodulen til aktiv prosjektfil */
         if (!AddFDataToPrj (&fdata))
         {
            #if __IBMC__ || __IBMCPP__
            DosFindClose (FindHandle);
            #endif
            RETURN_ERR;
         }

         if (job != PRJLINK)           /* Hvis kartet ikke bare skal linkes */
            if (!ScanFilesInPrj (job)) /* Utfr scan av aktiv/eneste kildefil*/
               RETURN_ERR;

         if (o_generate.gen != O_TEMP) /* Hvis mer skal gjres enn scan */
            if (!MakeMapFile ())       /* Produser selve kartfilen */
               RETURN_ERR;

         printf ("\n\n");              /* To linjeskift mellom hver kildefil */
      }

      #if __IBMC__ || __IBMCPP__
      rc = DosFindNext (FindHandle, (PVOID) &FFBuff, sizeof (FFBuff), &ulFindCount);
      #else
      done = findnext (&_ffblk);
      #endif
   }

   #if __IBMC__ || __IBMCPP__
   DosFindClose (FindHandle);
   #else
   #endif

   if (countf == 0)                    /* Hvis ingen filnavn er funnet totalt*/
      RETERR (1, path);                /* Ingen matchende filnavn */

   return (OK);                        /* Vellykket kartgenerasjon */
} /* MakeMapFiles (); */




int RemoveAllSmg ( void )
/* Date: February 08, 1996. By LEL.
   Delete all .smg-files. Call this upon termination of SM, if user want
   us to do it (via user options) */
{
   int c1;
   prjfeTYPE fdata;

   for (c1 = 1; c1 <= prjstat.filc; c1++) /* All sources in project */
   {
      if (!GetFDataFromPrj (c1, &fdata))  /* Get name of source file in projec*/
         RETURN_ERR;
      Remove (fdata.objn);                /* Delete this .smg-file */
   }

   return OK;
} /* RemoveAllSmg (); */




void CallWhenExit ( void )
/*
    Function: Allways call this function before termiante.
        Date: 1992. By LEL.
     Comment: To force a call of this function whenever the program
              terminate, add it to the quew of functions to call when
              exit by calling the standard ANSI C function 'atexit ()'.
*/
{
   if (o_various.del)                  /* If we shall delete all .smg-files  */
      RemoveAllSmg ();                 /* Delete all .smg-files of prj */

   #if __IBMC__ || __IBMCPP__
   _fcloseall ();
   #elif __GNUC__
   FCloseAll ();
   #else
   fcloseall ();
   #endif

   Remove (tmpn.funcal);               /* Temporary file of function calls   */
   Remove (tmpn.lininf);               /* Temporary file of line information */
   Remove (tmpn.funreg);               /* Temporary file of function register*/
   Remove (tmpn.idlist);               /* Temporary file of items            */
   Remove (tmpn.manobj);               /* Temporary main objectfile          */
   Remove (tmpn.sortfi);               /* Temporary file used when sorting   */
   Remove (tmpn.indexf);               /* Temporary file used to index data  */

   if (prjstat.temp)                   /* Is active project temporary?       */
   {
      if (prjstat.name[0])             /* If this filename has been created  */
         Remove (prjstat.name);        /* Yes, delete temoprary projectfile  */
      else
         Remove (tmpn.projec);         /* Delete default temporary projectf. */
   }
} /* CallWhenExit (); */




int CallWhenCtrlBrk ( void )
/*
    Function: Call this function whenever user press control-break on
              keyboard. Also call upon any critical errors, such as divide
              by zero.
        Date: September 22, 1992. By LEL.
     Returns: A dummy value 0.
     Comment: To force a call to this function when user press control-
              break, use the function 'ctrlbrk ()'.
*/
{
   char mapname [MAXPATH + 1];  /* Name of map file */

   Display (D_HEIGHEST,
            "\n"
            "\n"
            "Oups! <Control-C> or <Control-Break> detected.\n"
            "Cleaning up and terminating the program...\n");

   FCloseAll ();                /* Close all open files */

   MakeMapName (mapname);       /* Get name of the map file */
   Remove (mapname);            /* Delete unfinished map file */

   exit (-1);                   /* Call CallWhenExit() and abort the program */

   return (0);                  /* Return dummy 0, to satisfy caller */
} /* CallWhenCtrlBrk (); */




#if defined (__GNUC__) || defined (__WIN32__)
#pragma argsused
void CatchSignal ( int iType )
{
   signal (SIGINT, CatchSignal);  /* Reinstall the signal handler (!!!) */
   CallWhenCtrlBrk ();
}
#endif




#ifdef __OS2__
#pragma argsused
ULONG _cdecl ExceptRoutine ( PEXCEPTIONREPORTRECORD pExRep,
                             PEXCEPTIONREGISTRATIONRECORD pExReg,
                             PCONTEXTRECORD pContext, PVOID pVoid )
/* Exception handler for OS/2. This will be called whenever user
   press Ctrl-C or Ctrl-Break. */
{
   static int iIsHere = FALSE;

   if (pExRep->ExceptionNum == XCPT_SIGNAL)
   {
      if (!iIsHere)  /* Prevent reqursive exception handling upon exit (); */
      {
         iIsHere = TRUE;
         CallWhenCtrlBrk (); /* This statement will never return ... */
         iIsHere = FALSE;
      }
   }

   return (XCPT_CONTINUE_SEARCH);
}
#endif




int RunCLineVer ( char *param )
/*
    Function: Main interface to command line version of source map
              generator (SMG).
        Date: February, 4. 1993.
   Interface: param = Argument from DOS, converted to a single string.
     Returns: E/O.
*/
{
   int       c1 = 0;
   int       job;                      /* PRJMAKE/PRJBUILD/PRJLINK */
   prjfeTYPE fdata;                    /* Mellomlager for kildefildata */
   #ifndef REGISTERED
   char *pUnregistered = "THIS COPY ISN'T REGISTERED. PLEASE REGISTER!";
   int iC1, iChkSum;
   for (iC1 = 0, iChkSum = 0;
        iC1 < strlen (pUnregistered);
        iChkSum += pUnregistered[iC1++]);
   if (iChkSum != 3028)
   {
      printf ("This program has been hacked and is therefore unable to run! :-)\n");
      exit (-1);
   }
   printf ("%s\n", pUnregistered);
   #endif

   /* Vis hjelp om SMG dersom ingen parametere er oppgitt: */
   if (param[0] == '\0')
   {
      printf ("%s\n\n%s", _StrMSGCOPYRIGHT, _StrMSGUSAGE);
      return (OK);
   }

   ResetAllFNames ();                  /* Nullstiller filnavn-lager */
   ResetPrjStat ();                    /* Initier data om prosjekt */

   if (!SetDefaultOptions ())          /* Initier brukervalg til standard */
      RETURN_ERR;

   /* --- Copyright information --- */
   Display (D_HEIGHEST, "%s\n\n", _StrMSGCOPYRIGHT);

   /* Les standard valgfiler (frst fra hjemmekatalogen til sm.exe, deretter
      fra aktiv katalog hvis den er ulik hjemmekatalogen) */
   read_default_options ();

   if (!interpret_parameters (param, FALSE))  /* Translate options from user */
      RETURN_ERR;

   if (!SetTmpFNames ())               /* Set and create temporary filenames */
      RETURN_ERR;

   if (o_generate.met == O_LINK && o_generate.gen == O_MAP)
   {
      /* Conflictiong options: Can't link map without making .SMO-files! */
      RETERR (7, NULL);
   }

   if (prjstat.filc == 0)              /* Any source-files in project?       */
      return (OK);                     /* No, do not make a map              */

   while (c1++ < prjstat.filc)         /* Loop trough all sources in project */
   {
      if (!GetFDataFromPrj (c1, &fdata)) /* Get name of source file in projec*/
         RETURN_ERR;

      /* Construct name of objectfile from name of sourcefile */
      GetObjFileName (fdata.objn, fdata.name);

      if (!PutFDataToPrj (c1, &fdata)) /* Update project-info about source   */
         RETURN_ERR;
   }

   switch (o_generate.met)
   {
      case O_LINK:           /* Link existing objectfiles to map */
           job = PRJLINK;
           break;

      case O_MAKE:           /* Make only changed source-files */
           job = PRJMAKE;
           break;

      case O_BUILD:          /* Build all objectfiles, and mapfiles */
           job = PRJBUILD;
           break;

      default:               /* Unknown method, return error */
           RETERR (8, NULL);
   }

   if (prjstat.filc == 1)              /* If only one sourcefile in project  */
   {
      if (!GetFDataFromPrj (1, &fdata)) /* Get sourcefile info from project  */
         RETURN_ERR;

      if (IsWild (fdata.name))         /* Any wildchar in name of sourcefile */
      {
         /* Yes, make maps of wildchar-files */
         if (!MakeMapFiles (job, fdata.name))
            RETURN_ERR;
         return (OK);
      }
   }

   if (job != PRJLINK)
      if (!ScanFilesInPrj (job))
         RETURN_ERR;

   if (o_generate.gen != O_TEMP)       /* Ikke hvis bare mellomfil skal lages*/
      if (!MakeMapFile ())             /* Produser selve kartfilen */
         RETURN_ERR;

   if (prjstat.file)                   /* If project file is open */
   {
      FClose (prjstat.file);           /* Close it */
      prjstat.file = NULL;
   }

   return (OK);
} /* RunCLineVer (); */




int mainCLineVer ( int argc, char *argv[] )
/*
        Date: April, 7. 1992.
   Interface: argc = Number of strings in 'argv' from DOS.
              argv = Table of argument strings from DOS.
     Returns: E/O.
*/
{
   #ifdef __OS2__
   struct
   { PEXCEPTIONREGISTRATIONRECORD pLink;
     ULONG (_cdecl *pSysEH) (PEXCEPTIONREPORTRECORD,
                             PEXCEPTIONREGISTRATIONRECORD,
                             PCONTEXTRECORD, PVOID);
   } RegRec;                     /* Structure to pass to exception handler */
   #endif
   int  c1 = 1;
   char param [MAXUSERPARAML + 1];

   setbuf (stdout, NULL);        /* Unbuffered screen output */

   /* Katalog til SMG1 program-katalog */
   strcpy (progdir, GetPath (argv[0]));

   /* Oversett parametere fra kommandolinja, ved  skjte dem sammen
      til n streng: */
   param [0] = 0;
   while (c1 < argc)
   {
      if (strlen (param) + strlen (argv[c1]) < MAXUSERPARAML - 1)
      {
         strcat (param, argv[c1++]);
         strcat (param, " ");
      }
      else
      {
         RETERR (10, NULL);      /* To many arguments from DOS */
      }
   }

   #if defined (__GNUC__) || defined (__WIN32__)
   signal (SIGINT, CatchSignal);
   #elif __OS2__
   /* The DosSetExceptionHandler call will link the exception registration
      record into the chain for the thread */
   RegRec.pLink = 0;
   RegRec.pSysEH = ExceptRoutine; /* Pointer to the exception handler */
   DosSetExceptionHandler ((PEXCEPTIONREGISTRATIONRECORD) &RegRec);
   #else
   ctrlbrk (CallWhenCtrlBrk);          /* Sett ctrlbrk-handler */
   #endif

   atexit (CallWhenExit);              /* Kall denne funksjonen ved exit */

   if (!RunCLineVer (param))
   {
      #ifdef __OS2__
      DosUnsetExceptionHandler ((PEXCEPTIONREGISTRATIONRECORD) &RegRec);
      #endif
      RETURN_ERR;
   }

   #ifdef __OS2__
   /* Must deregister the exception handler of OS/2 because the exception
      handler is no longer valid after this function has returned (the handler
      is part of the call stack) */
   DosUnsetExceptionHandler ((PEXCEPTIONREGISTRATIONRECORD) &RegRec);
   #endif

   return (OK);
} /* mainCLineVer (); */




int main ( int argc, char *argv[] )
/* Return:
   0 = Success.
   -1 = Some other error.
   Else the returned error is a system level error code. */
{
   #ifdef __OS2__
   /* Source Mapper now initialize it self to a lower task priority than
   is set by default by the system. This is due to the fact that SM is
   very disk intensive, and thus will perform about as fast even
   if the priority is relatively low. */
   DosSetPriority (PRTYS_THREAD,        /* Change a single thread */
                   PRTYC_IDLETIME,
                   PRTYD_MAXIMUM,
                   0L);                 /* Assume current thread  */
   #endif

   ErrorClear ();
   Randomize ();        /* Randomize maker of temporary filenames */
   if (!mainCLineVer (argc, argv))
   {
      ErrorShow ();
      return (Error.errno ? Error.errno : -1);
   }

   return (0);
} /* main (); */

