/* ------------------------------------------------------------------------
 *
 *        File: smgflow.c
 *     Project: Source Mapper.
 *     Created: Mai 12, 1993.
 * Description: Sourcecode to include calling sequence info to mapfile.
 *
 * Copyright (C) 2000 Leif-Erik Larsen.
 * This file is part of the Source Mapper source package.
 * Source Mapper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, in version 2 as it comes in the
 * "COPYING" file of the XWorkplace main distribution.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ------------------------------------------------------------------------ */




#include "smg.h"




typedef struct
{
   char name [MAXFUNCN + 1];
   long lnr;
}
   levelTYPE;




/* Below is a structure of data used by the reqursive function
   'FlowCSubLevels ()'. This items is better stored in such a global
   structure, instead of passing them as arguments on each reqursive call
   to the function. This to prevent unneccessary use of the stack.
   This structure must be initialized before the function 'FlowCSubLevels ()'
   is called first time. */

static struct
{
   FILE     *mapf;                     /* Map file to include the list       */
   FILE     *tmpf;                     /* Stripped temporary objectfile      */
   char      topl [MAXFUNCN + 1];      /* Name of function in current level  */
   long      lnr;                      /* Number of next line in the list    */
   int       indent;                   /* Current number of indents in list  */
}
   req;




int FlowCIncludeToMap ( FILE *mapf )
/*
        Function: Include calling sequence info to mapfile.
   Programmed by: LEL
            Date: Mai, 12. 1993. 13:35.
       Interface: mapf = Map file to include info of calling sequences.
         Returns: E/O.
*/
{
   int        ok;
   FILE      *objf;                    /* Main objectfile                    */
   fposTYPE   fpos;                    /* Positions of blocks of data in objf*/
   prjfeTYPE  fdata;                   /* Data about source in project       */

   if (!prjstat.prjon)                 /* If no active project               */
   {
      return (ERROR);
   }

   /* =======================================================================*/
   /* List of calling sequences:                                             */
   /*                                                                        */
   /*                                                                        */

   WrtStdLinComStart (mapf, stdwidth, linechr2);
   FPrintF (mapf, "%s:\n\n\n", _StrMSGFLOWCLSTSTR);

   if (errno != 0)                     /* Any error when writing header?     */
      return (ERROR);

   if (prjstat.filc == 1)              /* If only one source in project      */
   {
      /* Get name of normal objectfile */
      if (!GetFDataFromPrj (1, &fdata))
         RETURN_ERR;

      /* Use normal objfile if only 1 source */
      if ((objf = FOpen (fdata.objn, "r+b")) == NULL)
         RETERR (16, tmpn.manobj);     /* Failed on open file! */
   }
   else
   {
      /* Use main objectfile if several files */
      if ((objf = FOpen (tmpn.manobj, "r+b")) == NULL)
         RETERR (16, tmpn.manobj);     /* Failed on open file! */
   }

   if (!GetFPosTYPE (objf, &fpos))     /* Read position of blocks of data    */
   {
      FClose (objf);                   /* Yes, so close objectfile before ret*/
      RETURN_ERR;
   }

   ok = FlowCIncludeToMap_ (mapf, objf, &fpos); /* Make calling sequence */

   FClose (objf);                      /* Close main objectfile              */

   return (ok);                        /* OK generate of function index      */
} /* FlowCIncludeToMap (); */




int IsFuncCalled ( FILE *objf, const char *funcn, fposTYPE *fpos, int *answ )
/*
        Function:
   Programmed by: LEL
            Date: Mai, 12. 1993. 15:03.
       Interface: objf  = Opened, main objectfile.
                  funcn = Name of function to test if called.
                  fpos  = Positions to blocks of data in objectfile.
                  answ  = Returns answer: 1 if function is called, else 0.
         Returns: E/O.
*/
{
   char caller [MAXFUNCN + 1];         /* Buffer for storing name of function*/
   long lnr;                           /* Buffer for storing dummy line numbr*/
   long currpos;                       /* Current position of objectfile     */

   *answ = 0;                          /* Default is that func is not called */

   /* Seek to start of function call list */
   FSeek (objf, fpos->funcal, SEEK_SET);

   currpos = fpos->funcal;             /* Init current position controller */
   while (currpos < fpos->idlst)       /* Next datablock is list of id's */
   {
      if (!FGetS0 (caller, MAXFUNCN, objf)) /* Read name of called function */
         return (ERROR);


      if (strcmp (funcn, caller) == 0) /* Compare called func. with func.name*/
      {
         *answ = 1;                    /* Yes, so function is called         */
         return (OK);                  /* OK test                            */
      }

      do                               /* Do a dummy seek past line numbers  */
      {
         if (!FGetL (objf, &lnr))      /* Read next line number */
            return (ERROR);
      }
      while (lnr != 0);                /* A 0L terminates block of callers   */

      currpos = FTell (objf);          /* Update current position controller */
   }

   return (OK);                        /* OK test. Function not called       */
} /* IsFuncCalled (); */




int FlowCIncludeToMap_ ( FILE *mapf, FILE *objf, fposTYPE *fpos )
/*
    Function: Include calling sequence info of a single sourcefile
              to mapfile.
        Date: Mai, 12. 1993. 13:40. LEL.
   Interface: mapf = Map file to include info of calling sequences.
              objf = Newly opened main objectfile.
              fpos = File positions to blocks of data in main objectfile.
     Returns: E/O.
*/
{
   int        answ;                    /* Store answers                      */
   int        ctopl;                   /* Number of top level functions      */
   long       c1;                      /* Counter 1                          */
   long       lnr = 1;                 /* Count line number in list          */
   FILE      *diff;                    /* File of top-level functions        */
   FILE      *tmpf;                    /* Temporary file of stripped list    */
   funcTYPE   func;                    /* Info on current function           */
   levelTYPE  level;

   /* Seeks for top-level functions */
   Display (D_HEIGH, "%s", _StrMSGSEEKTPLEVELF);

   /* Init list of non-caled/top-level */
   if (!DiffBConstruct (tmpn.sortfi, MAXFUNCN + 1))
   {
      DiffBDestruct ();                /* Terminate list of included funccals */
      RETURN_ERR;
   }

   c1 = 0;                             /* Reset controller of display-info */

   /* Get info about next function */
   while (FuncRBuffIndxGet (c1++, &func))
   {
      if (c1 % 4 == 0)                 /* Update userinfo on each 4 */
         Display (D_LOW, "%s: %ld%%", _StrMSGSEEKTPLEVELF, (c1 * 100) / FuncRBuffGetFNr ());

      /* Test if function is called */
      if (!IsFuncCalled (objf, func.name, fpos, &answ))
      {
         DiffBDestruct ();             /* Terminate table of non-called funcs */
         RETURN_ERR;
      }

      if (answ == 0)                   /* Is function called? */
      {
         if (!DiffBAddId (func.name))  /* No, so add name of function to list*/
         {
            DiffBDestruct ();          /* Terminate table of non-called funcs*/
            RETURN_ERR;
         }
      }
   }

   /* Seeks for top-level functions: 100% */
   Display (D_LOW, "%s: 100%%", _StrMSGSEEKTPLEVELF);

   ctopl = diffb.cid;                  /* Copy number of top-level functions */
   if (ctopl < 1)                      /* Any non-called function?           */
   {
      /* Warning: No top level function */
      Display (D_WARNING, "%s: %s ()", _StrMSGNOTOPLEVELFUNC, o_flow.deftoplev);

      /* Test if default name of top-level function is in function buffer:   */
      if (FuncRBuffIsFunc (o_flow.deftoplev))
      {
         if (!DiffBAddId (o_flow.deftoplev))
         {
            DiffBDestruct ();          /* Terminate table of non-called funcs*/
            RETURN_ERR;
         }
      }

      /* If default name of top-level function is not in function-buffer,
         then we know that this function is not defined in the project.
         Therefore, we have no top-level function to start flow-chart from,
         so do nothing more but return a dummy OK: */

      else                             /* Default top-level isn't in buffer  */
      {
         DiffBDestruct ();             /* Terminate table of non-called funcs*/
         return (OK);
      }
   }

   DiffBDestruct ();                   /* Terminate table of non-called funcs*/

   /*------------------------------------------------------------------------*/

   /* Open file of top-level functions */
   if ((diff = FOpen (tmpn.sortfi, "r+b")) == NULL)
      RETERR (11, tmpn.sortfi);        /* Failed on open temporary file! */

   if (!sortfile (diff, MAXFUNCN + 1)) /* Sort list of top-level functions   */
   {
      FClose (diff);
      RETERR (12, GetFNames (diff));   /* Failed on sort contents of file! */
   }

   /* Init list of allready included functions */
   if (!DiffBConstruct (tmpn.funcal, sizeof (levelTYPE)))
   {
      DiffBDestruct ();                /* Terminate list of included funccals*/
      FClose (diff);
      RETURN_ERR;
   }

   /* Open temporary strip file for writing */
   if ((tmpf = FOpen (tmpn.idlist, "w+b")) == NULL)
   {
      DiffBDestruct ();                /* Terminate list of included funccals*/
      FClose (diff);
      RETERR (11, tmpn.idlist);        /* Failed on open temporary file! */
   }

   /* Strip objectfile into 'tmpf' */
   if (!FlowCStripTmpf (objf, tmpf, fpos))
   {
      FClose (tmpf);
      DiffBDestruct ();                /* Terminate list of included funccals*/
      FClose (diff);
      RETURN_ERR;
   }

   /* Loop trough all top-level functions */
   for (c1 = 0; c1 < ctopl; c1++)
   {
      /* Read name of next top-level function */
      if (FRead (&func.name, MAXFUNCN + 1, 1, diff) != 1)
      {
         FClose (tmpf);
         DiffBDestruct ();             /* Terminate list of included funccals*/
         FClose (diff);
         return (ERROR);
      }

      strcpy (level.name, func.name);  /* Name of top level function         */
      level.lnr = lnr;                 /* Line number of calling sequence    */

      /* Add it to list of allready included */
      if (!DiffBAddId ((char *) &level))
      {
         FClose (tmpf);                /* Yes, so close files before return  */
         DiffBDestruct ();             /* Terminate list of included funccals*/
         FClose (diff);
         RETURN_ERR;
      }

      /* Include cal.-seq. for this top-level */
      if (!FlowCIncludeToMap__ (mapf, tmpf, func.name, &lnr))
      {
         FClose (tmpf);                /* Close files before return  */
         DiffBDestruct ();             /* Terminate list of included funccals*/
         FClose (diff);
         RETURN_ERR;
      }

      if (c1 < ctopl - 1)
      {
         FPrintF     (mapf, "\n\n");   /* New-lines between each top-level   */
         WritFLineLn (mapf, stdwidth, linechr1);
      }
   }

   FClose (tmpf);                      /* Close temporary strip-file         */
   DiffBDestruct ();                   /* Terminate list of included funccals*/
   FClose (diff);                      /* Close list of top-level functions  */

   /*------------------------------------------------------------------------*/

   if (o_flow.index != O_YES)          /* Shall we include index-register?   */
   {
      if (o_various.gencom == O_YES)   /* No, so terminate eventually comment*/
         WrtComEndLn (mapf);

      goto OK_RET;                     /* And jump past include of index     */
   }

   /* Re-open list of included functions */
   if ((diff = FOpen (tmpn.funcal, "r+b")) == NULL)
      RETERR (11, tmpn.funcal);        /* Failed on open temporary file! */

   Display (D_HEIGH, "%s", _StrMSGSORTFLOWIDXREG); /* Display info to user */

   if (!sortfile (diff, sizeof (levelTYPE))) /* Sort included functions */
   {
      FClose (diff);
      RETERR (12, GetFNames (diff));   /* Failed on sort contents of file! */
   }

   if (!FlowCInclIndx (mapf, diff))    /* Include index-register to map      */
   {
      FClose (diff);
      RETURN_ERR;
   }

   FClose (diff);

OK_RET:

   FPrintF (mapf, "\n\n\n");
   if (o_various.np_sect == O_YES)     /* Page break between map sections? */
      FPrintF (mapf, "%c", NPCHR);     /* Yes, so include new-page character */

   if (errno != 0)                     /* Any error on last I/O operations?  */
      return (ERROR);

   return (OK);
} /* FlowCIncludeToMap_ (); */




int FlowCStripTmpf ( FILE *objf, FILE *tmpf, fposTYPE *fpos )
/*
    Function: Strip info of function calls in file 'objf' into
              temporary file 'tmpf'. Whit this, we mean:
              Strip away all superfluous line number references so that
              no calles have more than one line number reference to
              each mother function.
        Date: Mai, 13. 1993. 10:40. By LEL.
   Interface: objf = Objectfile to strip.
              tmpf = Store stripped data in this file.
              fpos = File-positions to blocks of data in 'objf'.
     Returns: E/O.
*/
{
   long     line;                      /* Store line number of function-call */
   long     currpos;                   /* Current position in objectfile     */
   char     fname [MAXFUNCN + 1];      /* Name of current function-call      */
   char     last  [MAXFUNCN + 1];      /* Name of last stripped function-call*/
   funcTYPE func;                      /* Info of function                   */

   Display (D_HEIGH, "%s", _StrMSGFLOWGENSTRIP);

   FSeek (objf,fpos->funcal,SEEK_SET); /* Seek to start of function-calls    */
   currpos = fpos->funcal;             /* Init controller of file position   */
   while (currpos < fpos->idlst)       /* Loop to strip objf into 'tmpf'     */
   {
      FGetS0 (fname, MAXFUNCN, objf);  /* Read name of next function-call    */
      FPrintF (tmpf, "%s%c",           /* Update strip file with this caller */
                     fname, '\0');     /* Terminate the name with a '\0'-char*/

      if (errno != 0)                  /* Any error on last I/O?             */
      {
         FClose (tmpf);                /* Yes, so close temp. file before ret*/
         return (ERROR);
      }

      last[0] = '\0';                  /* Reset buffer of last stripped call */

      while (1)                        /* Loop to include and strip line-nrs */
      {
         FGetL (objf, &line);          /* Read next line number              */

         if (errno != 0)               /* Any error on last I/O?             */
         {
            FClose (tmpf);             /* Yes, so close temp. file before ret*/
            return (ERROR);
         }

         if (line <= 0)                /* A 0L indicate the end of the linnrs*/
            break;                     /* So, break the loop of line-reading */

         /* To whitch function does line belong? */
         if (FuncRBuffLNrGet (line, &func))
         {
            /* Is owner another than current func? */
            if (strcmp (func.name, fname) != 0)
            {
               /* Yes, so test if current functioncall is already called by
                  function */
               if (strcmp (last, func.name) == 0)
                  continue;            /* Don't include more than once       */

               strcpy (last,func.name);/* Remember name of last owner func.  */

               if (!FPutL (tmpf, line))
               {
                  FClose (tmpf);
                  return (ERROR);
               }
            }
         }
      }
      currpos = FTell (objf);          /* Update controller of file position */
      FPutL (tmpf, 0L);                /* Include a 0L to mark end of lines  */
   }

   return (OK);                        /* OK stripping                       */
} /* FlowCStripTmpf (); */




int FlowCIncludeToMap__ ( FILE *mapf, FILE *tmpf, const char *topl, long *lnr )
/*
    Function: Include calling sequence to mapfile.
        Date: Mai, 12. 1993. 16:45. By LEL.
   Interface: mapf = Map file to include info of calling sequences.
              tmpf = Stripped data of function calls.
              fpos = File positions to blocks of data in main objectfile.
              topl = Name of current top-level function.
              lnr  = Number of next line in the list.
     Returns: E/O.
*/
{
   int  ret;                           /* Value to return                    */
   int  indent = 0;                    /* Current level of indent            */
   static char str [MAXFUNCN + 5];

   sprintf (str, "%s ()", topl);       /* Include top level function to list */
   if (!FlowCDrawBox (mapf, *lnr, 0, str))
      RETURN_ERR;

   req.mapf   =  mapf;                 /* Init items used by reqursive func  */
   req.tmpf   =  tmpf;
   req.lnr    = (*lnr)++;
   req.indent =  indent;
   strcpy (req.topl, topl);

   ret = FlowCSubLevels ();            /* Pass into reqursive function       */

   *lnr = req.lnr + 1;                 /* Update counter of number of lines  */

   return (ret);
} /* FlowCIncludeToMap__ (); */




int FlowCSubLevels ( void )
/*
    Function: Include sub-levels of call-sequence from specified
              name of function ('topl').
        Date: Mai, 12. 1993. 18:40. By LEL.
   Interface: See declaration of global structure 'req' at top of file.
     Returns: E/O.
*/
{
   static int  washere = FALSE;
   char        fname [MAXFUNCN + 1];
   static char swap  [MAXFUNCN + 1];
   static char str   [MAXFUNCN + 35];
   long             currpos;
   long             line;
   static uint16    answ;
   static funcTYPE  func;
   static levelTYPE level;

   req.indent += 1;                    /* A new indent of call-sequence      */

   FSeek (req.tmpf, 0L, SEEK_SET);     /* Seek to start of file              */
   currpos = 0L;                       /* Init controller of current filepos */

   /* Info to user. Do this only once, to prevent the message from
      flashing due to reqursive calls to this function. */
   if (!washere)
   {
      washere = TRUE;
      Display (D_HEIGH, "%s", _StrMSGINCLFLOWC);
   }

   /* Loop to include sub-calls */
   while (FGetS0 (fname, MAXFUNCN, req.tmpf)) /* Name of next caller */
   {
      while (1)                        /* Loop to find sub-calls             */
      {
         FGetL (req.tmpf, &line);      /* Read next line number ref.         */
         if (line <= 0)                /* A 0L indicate the end of the linnrs*/
            break;                     /* So, break the loop of line-reading */

         currpos = FTell (req.tmpf);   /* Update controller of current f.pos.*/

         /* We allready know that the line is owned by a function, because
            we did such a test in function 'FlowCStripTmpf ()' when we
            searched and stripped line number references:                      */
         FuncRBuffLNrGet (line,&func); /* Read name of owner function        */

         /* If owner equal actual level-func (which is in req.topl) */
         if (strcmp (func.name, req.topl) == 0)
         {
            req.lnr += 1;              /* Count linenumber in list */

            DiffBIfId (fname, &answ);  /* See if sub-sequence allready incl. */
            if (answ == 0)             /* If not allready included */
            {
               /* Info to user */
               Display (D_LOW, "%s (%c-%ld)", _StrMSGINCLFLOWC, LOCLINCHR, req.lnr);

               /* Test if called function is in buffer of functions. If
                  so, then we know that the called function is a function
                  which is defined in some of the source files of the
                  project. Then we allso must include a sub-sequence of the
                  called function. If called function is not in buffer of
                  functions, then we know that the called function is a
                  function which is an external: */

               if (!FuncRBuffIsFunc (fname))
               {
                  /* Function is not in buffer, so mark it as an external */
                  if (o_flow.iIncExternals)
                  {
                     sprintf (str, "%s () %s", fname, _StrMSGEXTERNAL);
                     if (!FlowCDrawBox (req.mapf, req.lnr, req.indent, str))
                        RETURN_ERR;
                  }
                  else
                  {
                     req.lnr -= 1;     /* Don't count this skipped function */
                  }
                  continue;            /* Don't include sub-sequence(s) */
               }

               /* Called function is in buffer */
               sprintf (str, "%s ()", fname);
               if (!FlowCDrawBox (req.mapf, req.lnr, req.indent, str))
                  RETURN_ERR;

               /* So, prepare for add it to list of allready included
                  sub-sequences    */
               strcpy (level.name, fname);
               level.lnr = req.lnr;    /* The sub-sequence start at cur.line */

               /* Add it to list of allready included functions */
               if (!DiffBAddId ((char *) &level))
                  RETURN_ERR;

               strcpy (swap, req.topl);/* Swap 'req.topl' and 'fname'        */
               strcpy (req.topl,fname);
               strcpy (fname, swap);

               if (!FlowCSubLevels ()) /* Do reqursive sub-sequences         */
                  RETURN_ERR;

               strcpy (req.topl,fname);/* Restore current mother function    */

               /* Restore current file position */
               FSeek (req.tmpf, currpos, SEEK_SET);
            }

            else                       /* Sub-sequence allready included     */
            {
               /* So, get info of where it was incl. */
               if (!DiffBGetId (answ, (char *) &level))
                  RETURN_ERR;

               /* Reference of where sub-sequence is included */
               sprintf (str, "%s () ... [%c-%ld]", fname, LOCLINCHR, level.lnr);

               if (!FlowCDrawBox (req.mapf, req.lnr, req.indent, str))
                  RETURN_ERR;
            }
         }
      }

      currpos = FTell (req.tmpf);      /* Update controller of current f.pos.*/
   }

   req.indent -= 1;                    /* Finished this level of sub-sequence*/

   return (OK);                        /* OK include of sub-sequence         */
} /* FlowCSubLevels (); */




int FlowCDrawIndts ( FILE *mapf, int indent, char vertchr )
/*
    Function: Draw specified number of indents.
        Date: Mai, 13. 1993. 10:55. By LEL.
   Interface: mapf   = Map file of where to draw the indents.
              indent = Levele of indents.
              vertchr= Character to use when draw vertical lines.
     Returns: E/O.
*/
{
   while (indent-- > 0)
   {
      FPutC (mapf, vertchr);
      WritFLine (mapf, o_flow.indent - 1, ' ');
      if (errno != 0)
         return (ERROR);
   }

   return (OK);
} /* FlowCDrawIndts (); */




int FlowCDrawBox ( FILE *mapf, long lnr, int indent, const char *txt )
/*
    Function: Draw a single item of call sequence to mapfile.
              The item is drawed in respect to the options
              'o_flow.box' amd 'o_flow.graph', whitch is used to
              determind if
        Date: Mai, 13. 1993. 10:50. By LEL.
   Interface: mapf   = Map file to draw the item of call sequence.
              lnr    = Current line number/number of item.
              indent = Current level of indents.
              txt    = Item text (name of function, etc.)
     Returns: E/O.
     Comment: The item is drawed in respect to the options 'o_flow.box'
              and 'o_flow.graph', whitch is used to determind if
              items shall be drawed as boxes or as brief listing, and
              if we shall use graphical characters or not in the list.
*/
{
   int ret;
   static char graphchr[] = ">ͻȼ";
   static char textchr [] = "|+--||+-+++";

   if (o_flow.graph == O_YES)
      ret = FlowCDrawBox_ (mapf, lnr, indent, txt, graphchr);
   else
      ret = FlowCDrawBox_ (mapf, lnr, indent, txt, textchr);

   return (ret);
} /* FlowCDrawBox (); */




int FlowCDrawBox_ ( FILE *mapf, long lnr, int indent,
                    const char *txt, const char chrs[] )
/*
    Function: Kernel function of the function 'FlowCDrawBox ()'.
              Do the real draw of box of flowchart or (if call-sequence
              has the format of brief list) include single item to
              call-sequence.
        Date: Mai, 14. 1993. 00:28. By LEL.
   Interface: mapf   = Map file to draw the item of call sequence.
              lnr    = Current line number/number of item.
              indent = Current level of indents.
              txt    = Item text (name of function, etc.
              chrs   = Characters to use when eventually draw a box.
                       Se function 'FlowCDrawBox ()' where the two
                       different sets of characters are defined.

                       Graphical version of the string is:
                       chrs[0] = ''
                       chrs[1] = ''
                       chrs[2] = ''
                       chrs[3] = '>'
                       chrs[4] = ''
                       chrs[5] = ''
                       chrs[6] = ''
                       chrs[7] = ''
                       chrs[8] = ''
                       chrs[9] = ''
                       chrs[10]= ''
     Returns: E/O.
*/
{
   if (o_flow.format == O_LIST)        /* If user do not wish chart as boxes */
   {
      FPrintF        (mapf, "%c-%ld\t", LOCLINCHR, lnr);
      FlowCDrawIndts (mapf, indent, chrs[0]);
      FPrintF        (mapf, "%s\n", txt);

      if (errno != 0)
         return (ERROR);

      return (OK);
   }

   if (indent > 0)                     /* If level is not top level-function */
   {
      FPrintF        (mapf, "\t");
      FlowCDrawIndts (mapf, indent, chrs[0]);
      FPrintF        (mapf, "\n");
   }

   FPrintF        (mapf, "%c-%ld\t", LOCLINCHR, lnr);
   FlowCDrawIndts (mapf, indent, chrs[0]);

   FPutC          (mapf, chrs[6]);
   WritFLine      (mapf, strlen (txt) + 4, chrs[7]);
   FPrintF        (mapf, "%c\n", chrs[8]);

   FPrintF        (mapf, "\t");

   if (indent > 0)
   {
      FlowCDrawIndts (mapf, indent - 1, chrs[0]);
      FPutC          (mapf, chrs[1]);
      WritFLine      (mapf, o_flow.indent - 2, chrs[2]);
      FPutC          (mapf, chrs[3]);
   }

   if (indent > 0)
      FPutC       (mapf, chrs[4]);
   else
      FPutC       (mapf, chrs[5]);
   WritFLine      (mapf, 2, ' ');
   FPrintF        (mapf, "%s", txt);
   WritFLine      (mapf, 2, ' ');
   FPrintF        (mapf, "%c\n", chrs[5]);

   FPrintF        (mapf, "\t");
   if (indent > 0)
   {
      FlowCDrawIndts (mapf, indent - 1, chrs[0]);
      WritFLine      (mapf, o_flow.indent, ' ');
   }

   FPutC          (mapf, chrs[9]);
   WritFLine      (mapf, strlen (txt) + 4, chrs[7]);
   FPrintF        (mapf, "%c\n", chrs[10]);

   if (errno != 0)
      return (ERROR);

   return (OK);
} /* FlowCDrawBox_ (); */




int FlowCInclIndx ( FILE *mapf, FILE *funcs )
/*
        Function: Include index-register of call-sequences to mapfile.
   Programmed by: LEL
            Date: Mai, 14. 1993. 03:50.
       Interface: mapf  = File of where to include the index-register.
                  funcs = File with sorted functions which is included in
                          the call-sequence list.
         Returns: E/O.
*/
{
   long      c1 = 0;                   /* Count index number                 */
   levelTYPE func;                     /* Store current function index       */

   /* Be sure position is at start of file */
   if (!FSeek (funcs, 0L, SEEK_SET))
      return (ERROR);

   /* ---------------------------------------------------------------------- */
   /* Index register of calling sequences:                                   */
   /*                                                                        */

   FPrintF     (mapf, "\n\n\n");
   WritFLineLn (mapf, stdwidth, linechr1);
   FPrintF     (mapf, "%s:\n\n", _StrMSGIDXREGFLOW);

   if (errno != 0)                     /* Any error when writing header?     */
   {
      return (ERROR);
   }

   /* Display info to user */
   Display (D_HEIGH, "%s", _StrMSGINCLFLOWIDXREG);

   /* Loop and read all items in file */
   while (FRead (&func, sizeof (levelTYPE), 1, funcs) == 1)
   {
      Display (D_LOW, "%s: (%s ())", _StrMSGINCLFLOWIDXREG, func.name);

      /* F-nn   NameOfFunction ()    L-nnn */
      FPrintF (mapf, "\n%c-%ld\t%s ()\t%c-%ld",
                     FUNCTIONCHR, ++c1, func.name, LOCLINCHR, func.lnr);

      if (errno != 0)                  /* Any error when writing index?      */
         return (ERROR);
   }

   if (o_various.gencom == O_YES)      /* Shall we generate comment-blocks?  */
      WrtComEndLn (mapf);              /* Yes, so terminate current comment  */
   else
      FPrintF (mapf, "\n");            /* No, so only include a line feed    */

   if (errno != 0)                     /* Any error on last I/O operations?  */
      return (ERROR);

   return (OK);                        /* OK include of index register       */
} /* FlowCInclIndx (); */


