/* ------------------------------------------------------------------------
 *
 *        File: smglinf.c
 *     Project: Source Mapper.
 *     Created: January 10, 1993.
 * Description: Sourcecode to include sourcetext and lineinfo to mapfile.
 *
 * Copyright (C) 2000 Leif-Erik Larsen.
 * This file is part of the Source Mapper source package.
 * Source Mapper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, in version 2 as it comes in the
 * "COPYING" file of the XWorkplace main distribution.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ------------------------------------------------------------------------ */




#include "smg.h"




#define COLWIDTH 9                     /* Width of info-column to the left   */




int LInfoIncToMap ( FILE *mapf )
/*
        Function: Include source and lineinfo to map file.
   Programmed by: LEL
            Date: Januar, 10. 1993.
       Interface: mapf = Map file to include source list.
         Returns: E/O.
*/
{
   if (!prjstat.prjon)                 /* Any sources in current project?    */
   {
      return (ERROR);
   }

   if (prjstat.filc == 1)              /* If one single source in project    */
      return (LInfo1 (mapf));          /* Adjusted to single list of source  */
   else                                /* If more than one source in project */
      return (LInfo2 (mapf));          /* Adjust to list of several sources  */
} /* LInfoIncToMap (); */




int LInfoSourceHead ( FILE *mapf, int fnr )
/*
    Function: Include header of specified source file in project, to map.
        Date: Mai, 7. 1993. 01:55. By LEL.
   Interface: mapf = Map file to include header.
              fnr  = File number index of projectsource (1..).
     Returns: E/O.
*/
{
   if (prjstat.filc > 1)               /* If more than one source in project */
   {
      if (!wrt_totinfo1 (mapf, fnr))   /* Header as info of active source    */
      {
         RETURN_ERR;
      }
      return (OK);                     /* OK include of source header        */
   }

   errno = 0;                          /* Reset any error                    */

   WrtStdLinComStart (mapf, stdwidth, linechr2);
   FPrintF (mapf, "%s:", _StrMSGSOURCECODEINF);

   if (o_various.gencom)               /* Generate comment-blocks?           */
   {
      WrtComEnd (mapf);                /* Yes, so terminate block of comment */
   }

   FPrintF (mapf, "\n\n");             /* Write new-lines after header       */

   if (errno != 0)                     /* Any error when writing mapfile     */
   {
      return (ERROR);
   }

   return (OK);                        /* OK include of header               */
} /* LInfoSourceHead (); */




int LInfo1 ( FILE *mapf )
/*
        Function: Include source adjusted to a single sourcefile.
   Programmed by: LEL
            Date: Mai, 7. 1993. 01:50.
       Interface: mapf = Map file to include source list.
         Returns: E/O.
*/
{
   int       ret;
   long      glnr = 1;                 /* Number of next global line         */
   FILE     *objf;                     /* Objectfile                         */
   FILE     *srcf;                     /* Sourcefile                         */
   fposTYPE  fpos;                     /* Positions to blocks of data in objf*/
   prjfeTYPE fdata;

   if (!GetFDataFromPrj (1, &fdata))   /* Get info about sourcefile          */
   {
      RETURN_ERR;
   }

   /* Open objectfile */
   if ((objf = FOpen (fdata.objn, "r+b")) == NULL)
   {
      RETERR (16, fdata.objn);         /* Failed on open file! */
   }

   if (!GetFPosTYPE (objf, &fpos))     /* Get position to blocks of data     */
   {
      FClose (objf);
      RETURN_ERR;
   }

   FSeek (objf, fpos.lininf, SEEK_SET);/* Point to start of lineinfo         */

   if (!LInfoSourceHead (mapf, 1))     /* Include header of sourcefile       */
   {
      FClose (objf);
      RETURN_ERR;
   }

   /* Open the sourcefile */
   if ((srcf = FOpen (fdata.name, "rt")) == NULL)
   {
      FClose (objf);
      RETERR (23, fdata.name);         /* Failed on open source file! */
   }

   /* Include sourceline with info */
   ret = LInfoInclSource (mapf, objf, srcf, &glnr, fdata.finf.ltot, &fpos);

   FClose (srcf);                      /* Close current sourcefile           */
   FClose (objf);                      /* Close main objectfile              */

   return (ret);
} /* LInfo1 (); */




int LInfo2 ( FILE *mapf )
/*
        Function: Include source adjusted to several sourcefiles.
   Programmed by: LEL
            Date: Mai, 7. 1993. 01:50.
       Interface: mapf = Map file to include source list.
         Returns: E/O.
*/
{
   int       fc = 0;                   /* Count current file in project      */
   long      glnr = 1;                 /* Number of next global line         */
   FILE     *objf;                     /* Main objectfile                    */
   FILE     *srcf;                     /* Current sourcefile                 */
   fposTYPE  fpos;                     /* Positions to blocks of data in objf*/
   prjfeTYPE fdata;

   /* Open main objectfile */
   if ((objf = FOpen (tmpn.manobj, "r+b")) == NULL)
   {
      /* Failed on open temporary file! */
      RETERR (11, tmpn.manobj);
   }

   if (!GetFPosTYPE (objf, &fpos))     /* Get position to blocks of data     */
   {
      FClose (objf);
      RETURN_ERR;
   }

   FSeek (objf, fpos.lininf, SEEK_SET);/* Point to start of lineinfo         */

   while (fc++ < prjstat.filc)         /* All sources in project             */
   {
      if (!LInfoSourceHead (mapf, fc)) /* Include header of current source   */
      {
         FClose (objf);
         RETURN_ERR;
      }

      /* Get info about current sourcefile */
      if (!GetFDataFromPrj (fc, &fdata))
      {
         FClose (objf);
         RETURN_ERR;
      }

      /* Open current sourcefile */
      if ((srcf = FOpen (fdata.name, "rt")) == NULL)
      {
         FClose (objf);
         RETERR (23, fdata.name);      /* Failed on open sourcefile! */
      }

      /* Include sourceline with info */
      if (!LInfoInclSource (mapf, objf, srcf, &glnr, fdata.finf.ltot, &fpos))
      {
         FClose (objf);
         FClose (srcf);
         RETURN_ERR;
      }

      FClose (srcf);                   /* Close current sourcefile           */
   }

   FClose (objf);                      /* Close main objectfile              */

   return (OK);                        /* OK include of lineinfo             */
} /* LInfo2 (); */




int LInfoInclSource ( FILE *mapf, FILE *objf,
                      FILE *srcf, long *glnr, long lines, fposTYPE *fpos )
/*
    Function: Include specified source to mapfile.
        Date: Mai, 14. 1993. 15:20. By LEL.
   Interface: mapf = Map file to include source list.
              objf = Main objectfile.
              srcf = Current sourcefile.
              glnr = Number of next global line.
              lines= Number of lines in current sourcefile.
              fpos = Positions to blocks of data in main objectfile.
     Returns: E/O.
     Comment: The file 'objf' must point to start of block of lineinfo of
              specified source.
              The file 'srcf' must point to the start of file.
*/
{
   int      oldblock = 0;              /* Last number of block-indent        */
   long     llnr;                      /* Local line number                  */
   char     line[MAXINLIN+1];          /* Store current sourceline           */
   char     fname[MAXFILE+MAXEXT+1];   /* Name of current sourcefile         */
   scanTYPE scan;                      /* Scaninfo of current sourceline     */
   funcTYPE func;                      /* Info of next function              */

   /* Get name of current sourcefile */
   strcpy (fname, GetFName (GetFNames (srcf)));

   /* Includes sourceinfo: XXXXX.X */
   Display (D_HEIGH, "%s: %s", _StrMSGINCLINEINFO, fname);

   /* Find next func. after current line */
   if (!LInfoFindNextFunc (*glnr, &func))
   {
      RETURN_ERR;
   }

   for (llnr = 1;                      /* Loop trough all lines in sourcefile*/
        llnr <= lines;
        llnr++, (*glnr)++)             /* Allso count global line number     */
   {
      if (o_lineinfo.r_func)           /* If include of function info        */
      {
         if (func.blin != 0 &&         /* If any next function               */
             *glnr==func.blin_block)   /* If cur. line is start of next func.*/
         {
            /* Include all function info to map */
            if (!LInfoInclFuncInfo (mapf, objf, &func, fpos))
            {
               RETURN_ERR;
            }

            /* Find next func. after current line */
            if (!LInfoFindNextFunc (*glnr, &func))
            {
               RETURN_ERR;
            }
         }
      }

      if (llnr % 10 == 0)              /* Update userinfo on each 10. line   */
      {
         /* Includes sourceinfo: XXXXX.X (nnn) */
         Display (D_LOW, "%s: %s (%ld)", _StrMSGINCLINEINFO, fname, llnr);
      }

      /* Get scaninfo of current sourceline */
      FRead (&scan, sizeof (scan), 1, objf);

      FGetS (line, MAXINLIN, srcf);    /* Get current sourceline             */

      if (errno != 0)                  /* Any error on last I/O's?           */
      {
         return (ERROR);
      }

      if (scan.block > oldblock)       /* Inc. block only when block started */
      {
         oldblock = scan.block;        /* Remember last block-indent         */
         scan.block--;                 /* Adjust number of block-indent      */
      }
      else
      {
         oldblock = scan.block;        /* Remember last block-indent         */
      }

      /* Call kernel function */
      if (!LInfoInclSource_ (mapf, line, &scan, llnr, *glnr))
      {
         RETURN_ERR;
      }
   }

   ClrLin ();
   FPrintF (mapf, "\n\n\n");           /* Linefeeds between each sourcefile  */

   return (OK);                        /* OK include of sourceline/info      */
} /* LInfoInclSource () */




int LInfoInclSource_ ( FILE *mapf, const char *line, scanTYPE *scan,
                       long llnr, long glnr )
/*
        Function: Kernel function of 'LInfo2InclSource'. Include specified
                  line of source to mapfile.
   Programmed by: LEL
            Date: Mai, 19. 1993. 02:20.
       Interface: mapf = Map file to include source line.
                  line = Current source line.
                  scan = Scan info of current line.
                  llnr = Number of current local line.
                  glnr = Number of current global line.
         Returns: E/O.
*/
{
   int   c1;                           /* Counter 1                          */
   int   len;                          /* Length of new-page string          */
   char *ptr = (char *) line;          /* Pointer to current line            */

   /* Test if current line is a newpage-line only if both user has asked us
      to include newpages wherever such strings appear and if user has
      entered such a line.
   */

   if (o_various.np_str &&             /* If newpage when string for newpage */
       o_various.newpag[0])            /* And any string for newpage         */
   {
      len = strlen (o_various.newpag); /* Length of string for new-page      */

      for (c1 = 0; c1 < len; c1++)     /* Compare characters with newpage-str*/
      {
         if (*ptr++ != o_various.newpag[c1]) /* If strings are not equal */
            break;                     /* Then this is not a new-page line   */
      }

      if (c1 >= len)                   /* If the two strings was equal       */
         FPutC (mapf, NPCHR);          /* Then insert new-page character     */
   }

   if (scan->dirl &&                   /* Is current line a directive?       */
       o_various.gencom)               /* And generate comment-blocks        */
   {
      /* Since user has asked SMG to generate comment-blocks around info
         that SM includes, we can not write any info in front of any
         directive line. This is because any directiveline must start
         at line without any comment or source in front. Since we generates
         comments, we must suppose user wont to compile the generated map.
         Therefore, we now must include an empty block of info, instead of
         the normal block of info,
      */

      if (!LInfoEmpty (mapf))          /* Include an empty block of info     */
      {
         RETURN_ERR;
      }

      /* Include current sourceline */
      if (!PrintStrTab (mapf, line, o_various.twidth))
      {
         RETURN_ERR;
      }

      return (OK);                     /* OK include of current sourceline   */
   }

   if (o_various.gencom)               /* Shall we generate comment-blocks?  */
   {
      if (!scan->com1)                 /* Yes, but allready inside comment?  */
      {
         if (!WrtComStart (mapf))
            return (ERROR);
      }
      else
      {
         /* Since allready in com, write spaces */
         if (!WritFLine (mapf, 3, ' '))
            return (ERROR);
      }
   }

   /* L=Local line#, G=Global line# */
   FPrintF (mapf, "%c-%ld", prjstat.filc == 1 ? LOCLINCHR : GLOBLINCHR, glnr);

   /* Tabulate into next column */
   WritFLine (mapf, COLWIDTH - StrLenSL (glnr) - 2, ' ');

   if (prjstat.filc > 1 &&             /* If several sourcefiles             */
       o_lineinfo.r_llin)              /* and we shall include local linenr  */
   {
      /* then include number of local line  */
      FPrintF (mapf, "%c-%ld", LOCLINCHR, llnr);

      /* Tabulate into next column */
      WritFLine (mapf, COLWIDTH - StrLenSL (llnr) - 2, ' ');
   }

   if (scan->ocom)                     /* If whole line is comment */
   {
      if (o_lineinfo.r_coml)           /* and user has asked us to mark coml */
      {
         /* Mark line as comment-line */
         FPrintF   (mapf, "%s", _StrMSGONLYCOMENT);

         /* Tabulate into next column */
         WritFLine (mapf, COLWIDTH - strlen (_StrMSGONLYCOMENT), ' ');
      }
      else
      if (o_lineinfo.r_bloc)
      {
         WritFLine (mapf, COLWIDTH, ' '); /* Tabulate into next column */
      }
   }

   else
   if (scan->emp)                      /* If line is empty */
   {
      if (o_lineinfo.r_bloc ||         /* Report blocklevel */
          o_lineinfo.r_coml)           /* Report comment lines */
      {
         WritFLine (mapf, COLWIDTH, ' '); /* Tabulate into next column */
      }
   }

   else
   if (scan->block > 0 &&              /* If we are inside any level of block*/
       o_lineinfo.r_bloc)              /* and user has asked us to report it */
   {
      FPrintF (mapf, "%c", BLOCKCHR);  /* Write character for level of block */

      c1 = scan->block - 1;            /* Number of leading characters       */
      if (c1 > COLWIDTH - 3)           /* can not be greater than 6          */
      {
         c1 = COLWIDTH - 3;            /* Hold width inside 0-6              */
      }
      WritFLine (mapf, c1, '-');       /* Write leading characters           */

      /* Write level of block */
      FPrintF   (mapf, "%d", scan->block);

      /* Tabulate into next column */
      WritFLine (mapf, COLWIDTH - c1 - 2, ' ');
   }

   else
   if (o_lineinfo.r_bloc ||            /* If we use the mark-column          */
       o_lineinfo.r_coml)
   {
      WritFLine (mapf, COLWIDTH, ' '); /* Tabulate into next column          */
   }

   FPutC (mapf, '|');                  /* Include vertical separating line   */

   if (o_various.gencom)               /* Shall we generate comment-blocks?  */
   {
      if (!scan->com1)                 /* Yes, but allready inside comment?  */
      {
         if (!WrtComEnd (mapf))        /* No, so start block of comment      */
         {
            return (ERROR);
         }
      }
      else
      {
         /* Since allready in com, write spaces */
         if (!WritFLine (mapf, 3, ' '))
         {
            return (ERROR);
         }
      }
   }

   if (errno != 0)                     /* Any error on last I/O's?           */
   {
      return (ERROR);
   }

   /* Include current sourceline */
   if (!PrintStrTab (mapf, line, o_various.twidth))
   {
      return (ERROR);
   }

   return (OK);                        /* OK include of source/info          */
} /* LInfoInclSource_ (); */




int LInfoEmpty ( FILE *mapf )
/*
    Function: Write an empty info-column to the left of where source
              line is to be written.
        Date: Mai, 19. 1993. 12:30. By LEL.
   Interface: mapf = Map file to include the empty info.
     Returns: E/O.
*/
{
   if (o_various.gencom)               /* If user has asked us to gen. comm. */
   {
      WritFLine (mapf, 3, ' ');        /* Replace comment with spaces        */
   }

   WritFLine (mapf, COLWIDTH, ' ');    /* Replace global line number         */

   if (prjstat.filc > 1 &&             /* If several sourcefiles             */
       o_lineinfo.r_llin)              /* If we shall report number of locals*/
   {
      WritFLine (mapf, COLWIDTH, ' '); /* Replace local line number          */
   }

   if (o_lineinfo.r_bloc ||            /* If we use the mark-column          */
       o_lineinfo.r_coml)
   {
      WritFLine (mapf, COLWIDTH, ' '); /* Replace mark-column                */
   }

   FPutC (mapf, ' ');                  /* Replace '|'-character              */

   if (o_various.gencom)               /* If user has asked us to gen. comm. */
   {
      WritFLine (mapf, 3, ' ');        /* Replace comment with spaces        */
   }

   if (errno != 0)                     /* Any error on last I/O's?           */
   {
      return (ERROR);
   }

   return (OK);                        /* OK include of empty info           */
} /* LInfoEmpty (); */




int LInfoFindNextFunc ( long curlnr, funcTYPE *func )
/*
    Function: Find next function after specified global line number.
        Date: Mai, 19. 1993. 15:20. By LEL.
   Interface: curlnr = Start seek from this global line number.
              func   = Store eventually found function here.
     Returns: E/O.
     Comment: If there are no more functions after that global line
              number, then return 'func->blin' with the value 0.
              Else this item consist of the true starting line number
              of the found next function.
*/
{
   long c1;                            /* Counter 1                          */
   long fnr;                           /* Number of functions                */
   long nearest = 0;                   /* Nearest found line number of next  */

   fnr = FuncRBuffGetFNr ();           /* Get number of functions in buffer  */

   for (c1 = 0; c1 < fnr; c1++)        /* Seek trough all functions          */
   {
      FuncRBuffIndxGet (c1, func);     /* Get function which has this index  */

      if (curlnr <= func->blin)        /* If this function is after linenr   */
      {
         /* If no nearest is found yet or this function is nearer */
         if (nearest == 0 || func->blin < nearest)
         {
            nearest = func->blin;      /* then this is the nearest found now */
         }
      }
   }

   if (nearest == 0)                   /* If no nearest function was found   */
   {
      func->blin = 0;                  /* No more functions after that line  */
   }

   /* Get function who own found nearest  */
   if (FuncRBuffLNrGet (nearest, func) == 0) /* If no function own that line */
   {
      func->blin = 0;                  /* No more functions after that line  */
   }

   return (OK);                        /* Ok seek for next function          */
} /* LInfoFindNextFunc (); */




int LInfoInclFuncInfo ( FILE *mapf, FILE *objf,
                        funcTYPE *func, fposTYPE *fpos )
/*
    Function: Include all function info of specified function to mapfile.
        Date: Mai, 19. 1993. 15:54. By LEL.
   Interface: mapf = Mapfile of where to include the function info.
              objf = Main objectfile.
              func = Info about function.
              fpos = Positions to blocks of data i main objecfile.
     Returns: E/O.
     Comment: Before this function return, the position of the file
              'objf' will be restored back to the position it was before
              this function was called.
              But if any error, this function return without any
              respect to the original position of main objectfile.
*/
{
   int      infyes    = 0;             /* Used to test if any general info   */
   int      calyes    = 0;             /* Used to test if list of callers    */
   int      anycaller = 0;             /* 1 if any caller of the function    */
   long     oldpos;                    /* Original position of 'objf'        */
   long     curpos;                    /* Current position of 'objf'         */
   long     lnr;                       /* Linenumber of where call is done   */
   char     fname [MAXFUNCN + 1];      /* Name of called function            */
   char     oldcaller [MAXFUNCN + 1];  /* Name of previous caller            */
   funcTYPE caller;                    /* Function which do the call         */

   oldcaller [0] = '\0';               /* Reset name of previous caller      */

   /*------------------------------------------------------------------------*/
   if (o_lineinfo.c_ltot ||            /* If include of any general info     */
       o_lineinfo.c_empl ||
       o_lineinfo.c_coml ||
       o_lineinfo.c_lcod ||
       o_lineinfo.c_com)
   {
      infyes = 1;                      /* Remeber that some general info, yes*/
   }

   if (o_lineinfo.calist)              /* If include list of callers         */
   {
      calyes = 1;                      /* Remember that list of callers, yes */
   }

   if (!infyes && !calyes)             /* If neither gen. info nor callers   */
   {
      return (OK);                     /* Then do nothing but return         */
   }

   if (o_various.gencom)               /* If generate comment blocks         */
   {
      WrtComStartLn (mapf);            /* Start block of comment             */
   }
   else
   {
      FPrintF (mapf, "\n");            /* Empty line rather than comment */
   }

   if (errno != 0)                     /* Any error on last I/O's?           */
   {
      return (ERROR);
   }

   /*------------------------------------------------------------------------*/
   if (infyes)                         /* If include any general info        */
   {
      LInfoEmpty (mapf);               /* Tabulate into area of sourcecode   */

      /* Write header: General info...  */
      FPrintF (mapf, "%s '%s ()':\n\n", _StrMSGGENERALFUNCINF, func->name);

      if (o_lineinfo.c_ltot)           /* If report total number of lines    */
      {
         LInfoEmpty (mapf);            /* Tabulate into area of sourcecode   */
         FPrintF (mapf, "%s: %u\n", _StrMSGFUNCINF_LINTOT, func->lines);
      }

      if (o_lineinfo.c_empl)           /* If report number of empty lines    */
      {
         LInfoEmpty (mapf);            /* Tabulate into area of sourcecode   */
         FPrintF (mapf, "%s: %u\n", _StrMSGFUNCINF_LINEMP, func->empl);
      }

      if (o_lineinfo.c_coml)           /* If report number of comment lines  */
      {
         LInfoEmpty (mapf);            /* Tabulate into area of sourcecode   */
         FPrintF (mapf, "%s: %u\n", _StrMSGFUNCINF_LINCOM, func->coml);
      }

      if (o_lineinfo.c_lcod)           /* If report number of lines with code*/
      {
         LInfoEmpty (mapf);            /* Tabulate into area of sourcecode   */
         FPrintF (mapf, "%s: %u\n", _StrMSGFUNCINF_LINCOD, func->codl);
      }

      if (o_lineinfo.c_com)            /* If report number of comment blocks */
      {
         LInfoEmpty (mapf);            /* Tabulate into area of sourcecode   */
         FPrintF (mapf, "%s: %u\n", _StrMSGFUNCINF_CBLOCS, func->com);
      }

      if (calyes)
      {
         FPrintF (mapf, "\n");         /* Empty line before list of callers */
      }

      if (errno != 0)                  /* Any error on last I/O's?           */
      {
         return (ERROR);
      }
   }

   /*------------------------------------------------------------------------*/
   if (calyes)
   {
      LInfoEmpty (mapf);               /* Tabulate into area of sourcecode   */

      /* Write header: List of callers... */
      FPrintF (mapf, "%s '%s ()':\n\n", _StrMSGLISTOFCALLERS, func->name);

      oldpos = FTell (objf);           /* Remember original position of file */

      /* Start of datablock of function calls */
      FSeek (objf, fpos->funcal, SEEK_SET);

      curpos = fpos->funcal;           /* Init current position controller   */
      while (curpos < fpos->idlst)     /* Next datablock is identifiers      */
      {
         /* Read next sourceline from sourcefile */
         FGetS0 (fname, MAXFUNCN, objf);

         if (errno != 0)               /* Any error on last I/O's?           */
         {
            return (ERROR);
         }

         /* If call-name not equal func.name */
         if (strcmp (fname, func->name) != 0)
         {
            while (1)                  /* Do a dummy seek past line numbers  */
            {
               /* Read next line number */
               if (!FGetL (objf, &lnr))
               {
                  return (ERROR);
               }

               if (lnr == 0L)          /* Any 0L terminate list of line nr.  */
                  break;               /* So stop read of line numbers       */
            }

            curpos = FTell (objf);     /* Update current position of file    */

            if (errno != 0)            /* Any error on last I/O's?           */
            {
               return (ERROR);
            }

            continue;                  /* And continue read next call-name   */
         }

         /* Now, we have found the block of calls to specified functionname
            and the filepointer points to the line numbers of where each
            call is done. Therefore, start read those line numbers, and
            find which functions who own those lines.
         */

         while (1)                     /* Loop through all line numbers */
         {
            if (!FGetL (objf, &lnr))   /* Read next line number */
               return (ERROR);

            /* Any 0L terminates the list of line numbers */
            if (lnr == 0L)
               break;

            /* Which function own this line? If no function owns this line
               then continue reading next. This should really never happen,
               but in case... */
            if (FuncRBuffLNrGet (lnr, &caller) == 0)
               continue;

            /* Don't include same caller twice, so continue if not a
               new caller */
            if (strcmp (caller.name, oldcaller) == 0)
               continue;

            /* Remember current caller, for next test */
            strcpy (oldcaller, caller.name);

            anycaller = 1;             /* Now, at least one caller of func.  */

            LInfoEmpty (mapf);         /* Tabulate into area of sourcecode   */

            /* Name of function that do the call */
            FPrintF (mapf, "%s ()\t%c-%ld\n",
                           caller.name,
                           prjstat.filc > 1 ? GLOBLINCHR : LOCLINCHR,
                           caller.blin);

            if (errno != 0)            /* Any error on last I/O's?           */
            {
               return (ERROR);
            }
         }

         curpos = FTell (objf);        /* Update current position controller */

         if (errno != 0)               /* Any error on last I/O's?           */
         {
            return (ERROR);
         }
      }

      FSeek (objf, oldpos, SEEK_SET);  /* Restore to original position       */

      if (!anycaller)                  /* If no caller of that function      */
      {
         LInfoEmpty (mapf);            /* Tabulate into area of sourcecode   */
         FPrintF (mapf, "%s\n", _StrMSGNOBODYCALLFUNC); /* No caller... */
      }

      if (errno != 0)                  /* Any error on last I/O's?           */
      {
         return (ERROR);
      }
   }

   /*------------------------------------------------------------------------*/

   if (o_various.gencom)               /* If generate comment blocks         */
   {
      FPrintF (mapf, "*/\n");          /* Terminate block of comment         */
   }
   else
   {
      FPrintF (mapf, "\n");            /* Replace comment with empty line    */
   }

   if (errno != 0)                     /* Any error on last I/O's?           */
   {
      return (ERROR);
   }

   return (OK);                        /* OK include of function info        */
} /* LInfoInclFuncInfo (); */

