/* ------------------------------------------------------------------------
 *
 *        File: smgscanf.c
 *     Project: Source Mapper.
 *     Created: Mai 19, 1992.
 * Description: Kildekode for  scanne en enkelt kildefil.
 *
 * Copyright (C) 2000 Leif-Erik Larsen.
 * This file is part of the Source Mapper source package.
 * Source Mapper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, in version 2 as it comes in the
 * "COPYING" file of the XWorkplace main distribution.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ------------------------------------------------------------------------ */




#include "smg.h"




/* Lovlige tegn i funksjon og identifikator, ved syntaks for C: */
char *legal = "_"
              "0123456789"
              "abcdefghijklmnopqrstuvwxyz"
              "ABCDEFGHIJKLMNOPQRSTUVWXYZ";


/* Nkkelord, ved syntaks for C: */
char *keywords[] =
{
   /* ANSI standard C: */
   "auto",          "break",            "case",          "char",
   "const",         "continue",         "default",       "do",
   "double",        "else",             "enum",          "extern",
   "float",         "for",              "goto",          "if",
   "int",           "long",             "register",      "return",
   "short",         "signed",           "sizeof",        "static",
   "struct",        "switch",           "typedef",       "union",
   "unsigned",      "void",             "volatile",      "while",

   /* Turbo C: */
   "asm",           "__ss",             "interrupt",     "__cs",
   "cdecl",         "near",             "__ds",          "far",
   "pascal",        "__es",             "huge",

   /* Turbo C++ Versjon 3.1: */
   "_asm",          "__asm",            "_cdecl",        "__cdecl",
   "class",         "_cs",              "delete",        "_ds",
   "_es",           "_export",          "__export",      "_far",
   "__far",         "_fastcall",        "__fastcall",    "friend",
   "_huge",         "__huge",           "inline",        "_interrupt",
   "__interrupt",   "_loadds",          "__loadds",      "_near",
   "__near",        "new",              "operator",      "_pascal",
   "__pascal",      "private",          "protected",     "public",
   "_saveregs",     "__saveregs",       "_seg",          "__seg",
   "_ss",           "template",         "this",          "virtual",
};




#define MAXKEYWORD 83




/*
** Filene nedenfor er gjordt globale for  slippe  oppgi dem som parameter
** til alle de funksjonene som skal bruke dem:
*/
FILE *tempf;        /* For lagring av mellomfil */
FILE *sourcef;      /* Aktiv kildefil */
FILE *funcalf;      /* Midlertidig funksjonskall-liste */
FILE *lininff;      /* Midlertidig linjeinfo/totalinfo */
FILE *funregf;      /* Midlertidig funksjonsregister */
FILE *idlistf;      /* Midlertidig identifikatorliste */




static void DisplayCurrentLNr ( long iLNr )
/* Display line number of which source line containing the error.
   Created: February 04, 1996. By Leif-Erik Larsen. */
{
   Display (D_LOW, "%s (%ld)", _StrMSGINTERPRETLIN, iLNr);
} /* DisplayCurrentLNr (); */




int ifkeyword ( char *word )
/* Opprettet: Torsdag 28. mai 1992.
   Parameter: "word" Ord som skal skjekkes.
   Retur    : TRUE hvis "word" er et nkkelord, eller FALSE.
   Beskriv  : Sjekker om oppgitt ord er et nkkelord for C-syntaks
              eller ikke.
*/
{
   int t1;

   for (t1 = 0; t1 < MAXKEYWORD; t1++)
   {
      if (strcmp (keywords[t1], word) == 0)
         return (TRUE);
   }
   return (FALSE);
} /* ifkeyword (); */




int scan_sourcef ( char *sfnam, char *tfnam )
/* Opprettet: Onsdag 10. juni 1992.
   Parameter: "sfnam" Navn/katalog til kildefil som skal tolkes.
              "tfnam" Navn/katalog hvor endelig mellomfil skal lagres.
   Retur    : E/O.
   Beskriv  : Scanner oppgitt kildefil og produserer midlertidige
              filer av denne. Deretter kobles de midlertidige filene sammen
              til den endelige mellomfile. Denne funksjonen kaller
              "scan_sourcef_ ()" og utfrer ndvendige rutiner ved eventuell
              returnert feilkode. Dessuten informerer denne funksjonen
              bruker om hvilken kildefil som tolkes for yeblikket.
*/
{
   int ok;                             /* Mellomlagring av returkode */

   Display (D_HEIGHEST, "%s:\n", sfnam);

   if ((sourcef = FOpen (sfnam, "rt")) == NULL)
   {
      RETERR (23, sfnam); /* Failed on open source file! */
   }

   if ((tempf = FOpen (tfnam, "wb")) == NULL)
   {
      FClose (sourcef);
      RETERR (16, tfnam); /* Failed on open file! */
   }

   if ((funcalf = FOpen (tmpn.funcal, "w+b")) == NULL)
   {
      FClose (sourcef);
      FClose (tempf);
      RETERR (11, tmpn.funcal); /* Failed on open temporary file! */
   }

   if ((lininff = FOpen (tmpn.lininf, "w+b")) == NULL)
   {
      FClose (sourcef);
      FClose (tempf);
      FClose (funcalf);
      RETERR (11, tmpn.lininf); /* Failed on open temporary file! */
   }

   if ((funregf = FOpen (tmpn.funreg, "w+b")) == NULL)
   {
      FClose (sourcef);
      FClose (tempf);
      FClose (funcalf);
      FClose (lininff);
      RETERR (11, tmpn.funreg); /* Failed on open temporary file! */
   }

   if ((idlistf = FOpen (tmpn.idlist, "w+b")) == NULL)
   {
      FClose (sourcef);
      FClose (tempf);
      FClose (funcalf);
      FClose (lininff);
      FClose (funregf);
      RETERR (11, tmpn.idlist); /* Failed on open temporary file! */
   }

   ok = scan_sourcef_ (sfnam);

   FClose (sourcef);
   FClose (tempf);
   FClose (funcalf);
   FClose (lininff);
   FClose (funregf);
   FClose (idlistf);

   if (ok != OK)
      Remove (tfnam);    /* Remove unfinished .smg file */

   return (ok);
} /* scan_sourcef (); */




int scan_sourcef_ ( char *sfnam )
/* Opprettet: Onsdag 10. juni 1992.
   Parameter: "sfnam" Navn/katalog til kildefil som skal tolkes.
   Retur    : E/O.
   Beskriv  : Se funksjonen ovenfor: "scan_sourcef ()".
*/
{
   scanTYPE  scan;                 /* Info om aktiv linje under tolking */
   funcTYPE  func;                 /* Info om potensiell/bekreftet funksjon */
   finfTYPE  finf;                 /* Info om aktiv kildefil under tolking */
   fcalTYPE  fcal;                 /* Info om funksjonskall */
   idlstTYPE idlst;                /* Info om element i identifikatorliste */
   char      inlin [MAXINLIN + 2]; /* Buffer for innlesing av hel linje */

   reset_scantype  (&scan);
   reset_finftype  (&finf);
   reset_functype  (&func);
   reset_fcaltype  (&fcal);
   reset_idlsttype (&idlst);

   Display (D_HEIGH, "%s", _StrMSGINTERPRETLIN);

   while (FGetS (inlin, MAXINLIN, sourcef) == OK)
   {
      if (strlen (inlin) > MAXINLIN)
         RETERR (20, GetFNames (sourcef)); /* For lang linje i kildefil! */

      if (!intpret_line (inlin, &scan, &func, &finf, &idlst))
         RETURN_ERR;

      /* Lagrer linjeinfo til midlertidig fil for dette: */
      if (FWrite (&scan, sizeof (scanTYPE), 1, lininff) != 1)
         return (ERROR);

      /* Lagring av midlertidig data om funksjonskall, identifikatorer og
         funksjonsregister gjres internt i funksjonen "interpret_line()". */

      if (finf.ltot % 21 == 0)         /* Oppdater info for hver 21. */
         DisplayCurrentLNr (finf.ltot);
   }

   /* Lagre siste identifikator hvis potensiell identifikator: */
   if(idlst.name[0])
   {
      /* Lagrer element til id-liste i midlertidig fil for dette: */
      if (FWrite (&idlst, sizeof (idlstTYPE), 1, idlistf) != 1)
         return (ERROR);
      finf.cid++;                      /* Teller identifikatorer i kildefil */
   }

   if (errno != 0)                     /* Feil ved lesing av linje fra kilde*/
      return (ERROR);

   DisplayCurrentLNr (finf.ltot);

   FClose (sourcef);

   if (!ConnectTempFiles (sfnam, &finf))
      RETURN_ERR;

   return (OK);
} /* scan_sourcef_ (); */




void reset_scantype ( scanTYPE *scan )
/* Opprettet: Tirsdag 26. mai 1992.
   Parameter: "scan" Scaninfo-strukturen som skal resettes.
   Beskriv  : Resetter struktur som lagrer informasjon om aktiv linje under
              tolking av kildefil.
              Denne funksjonen m kalles mellom hver kildefil som skal tolkes.
*/
{
   scan->quot  = FALSE;
   scan->drop  = FALSE;
   scan->emp   = FALSE;
   scan->code  = FALSE;
   scan->ocom  = FALSE;
   scan->dirl  = FALSE;
   scan->com1  = FALSE;
   scan->intfc = FALSE;
   scan->comb  = 0;
   scan->brack = 0;
   scan->block = 0;
} /* reset_scantype (); */




void reset_functype ( funcTYPE *func )
/* Opprettet: Tirsdag 26. mai 1992.
   Parameter: "func" Funksjoninfo-struktur som skal resettes.
   Beskriv  : Resetter struktur som lagrer informasjon om funksjon under
              tolking av kildefil.
              Denne funksjonen m kalles mellom hver funksjon som blir
              bekreftes og avsluttet under tolkingen.
*/
{
   memset (func->name, 0, MAXFUNCN + 1);
   func->fok        = FALSE;
   func->blin       = 0;
   func->blin_block = 0;
   func->fcal       = 0;
   func->lines      = 0;
   func->empl       = 0;
   func->coml       = 0;
   func->codl       = 0;
   func->com        = 0;
   func->dirl       = 0;
} /* reset_functype (); */




void reset_fcaltype ( fcalTYPE *fcal )
/* Opprettet: Torsdag 28. mai 1992.
   Parameter: "fcal" Funksjonskallinfo-strukturen som skal resettes.
   Beskriv  : Resetter struktur som lagrer informasjon om enkeltstende
              funksjonskall under tolking av kildefil.
              Denne funksjonen m kalles mellom hvert registrerte
              bekreftede funksjonskall.
*/
{
   memset (fcal->name, 0, MAXFUNCN + 1);
   fcal->lnr = 0;
} /* reset_fcaltype (); */




void reset_finftype ( finfTYPE *finf )
/* Opprettet: Tirsdag 26. mai 1992.
   Parameter: "finf" Struktur for lagring av global info om kildefil.
   Beskriv  : Resetter struktur som lagrer global info om kildefil.
              Denne funksjonen m kalles mellom hver kildefil som skal
              tolkes.
*/
{
   finf->ltot = 0;
   finf->elin = 0;
   finf->coml = 0;
   finf->codl = 0;
   finf->com  = 0;
   finf->func = 0;
   finf->dirl = 0;
   finf->fcal = 0;
   finf->cid  = 0;
} /* reset_finftype (); */




void reset_idlsttype ( idlstTYPE *idlst )
/* Opprettet: Sndag 31. mai 1992.
   Parameter: "idlst" Struktur for lagring av id-liste-element.
   Beskriv  : Resetter struktur som lagrer info om elementer i
              liste over identifikatorer.
*/
{
   memset (idlst->name, 0, MAXFUNCN + 1);
   idlst->lnr  = 0;
} /* reset_idlsttype (); */




int intpret_line ( char *line, scanTYPE *scan, funcTYPE *func,
                   finfTYPE *finf, idlstTYPE *idlst )
/* Opprettet: Tirsdag 26. mai 1992.
   Parameter: "line"  Linjen som skal tolkes.
              "scan"  Struktur for lagring av resultater av scanningen.
              "func"  Struktur for lagring av info om funksjon.
              "finf"  Struktur for lagring av funksjonsdata.
              "idlst" Struktur for lagring av identifikatorliste.
   Retur    : E/O.
   Beskriv  : Tolker linjen "line" og setter strukturen "scan" i forrhold til
              resultatet av tolkingen, samt lagrer data om funksjoner og
              funksjonskall etter hvert som slik info samles i linjene
              som tolkes. Det forutsettes at linjen som skal tolkes er
              skrevet med syntaks for C.

              Bde "scan", "finf", "idlst" og "func" -strukturene m resettes
              utenfor denne funksjnen, mellom hver kildefil som skal tolkes.

              Lagring av liste over funksjonskall til midlertidig fil gjres
              i denne funksjonen. I denne forbindelse forutsettes det at
              den globale filen "funcalf" allerede er pen og klar for
              skriving.

              Tilsvarende gjelder for midlertidig fil for lagring av
              funksjonsregsiter, og fil for lagring av identifikatorliste.
              Det forutsettes at filene "funregf" og "idlistf" allerede
              er pnet og klar for skriving. Det forutsettes dessuten at
              "finf" og "idlst" resettes fr denne funksjonen kalles ved
              frste linje i hver kildefil.
*/
{
   register int t1;                    /* Ved dannelse av funksjonsnavn */
   char *res, *start = line;           /* For lagring av midlertidig posisjon*/
   fcalTYPE fcal;                      /* For lagring av funksjonskallinfo */

   scan->drop = FALSE;                 /* Aldri anfrselniv over fler linjer*/
   scan->quot = FALSE;                 /* Aldri gseniv over fler linjer */
   scan->ocom = TRUE;                  /* Utgangspunkt er at linjen er koment*/
   scan->emp  = TRUE;                  /* FALSE hvis andre karakterer enn ' '*/
   scan->code = FALSE;                 /* TRUE hvis kode i linjen */
   scan->dirl = FALSE;                 /* TRUE hvis linjen er direktivlinje*/

   finf->ltot++;                       /* Ny linje i aktiv kildefil */

   if(func->name[0])                   /* Hvis innenfor funksjonshode */
      func->lines++;                   /* Teller antall linjer i funksjon */

   if (scan->comb)                     /* Hvis 1. tegn i linje er i kom.blokk*/
      scan->com1 = TRUE;
   else
      scan->com1 = FALSE;

   for (; *line; line++)               /* Alle karakterer skal tolkes */
   {
      if(*line == '#' &&               /* # innleder alle direktiver */
         !scan->quot &&                /* Kun hvis utenfor streng mellom " " */
         !scan->drop)                  /* Kun hvis utenfor annfrselstegnniv*/
      {
         finf->dirl++;                 /* Teller direktivlinjer i aktiv kilde*/
         scan->dirl = TRUE;            /* Bekrefter direktiv i linje */
         scan->code = TRUE;            /* Bekrefter kode i aktiv linje */
         scan->emp = FALSE;            /* Bekrefter annet tegn enn ' ' funnet*/
         if (func->fok)                /* Hvis inne i bekreftet funksjon */
            func->dirl++;              /* Teller direktivlinjer i funksjon */
         break;                        /* Avbryt lkke for sk i linje */
      }

      if(*line != ' ' &&               /* Annet tegn enn ' ' er registrert */
         *line != '\t' &&              /* Tabulator-tegn */
         *line != '\n')                /* Linjeskift */
      {
         scan->emp = FALSE;            /* Bekrefter annet tegn enn ' ' funnet*/
         if(!scan->comb &&             /* Hvis utenfor komentarblokk */
            !scan->dirl &&             /* Hvis linjen ikke er direktiv */
            !(*line == '/' &&          /* Innledende komentarklamme... */
              *(line + 1) == '/') &&   /* ...regnes ikke som kode */
            !(*line == '/' &&          /* Innledende komentarklamme... */
              *(line + 1) == '*'))     /* ...regnes ikke som kode */
         {
            scan->code = TRUE;         /* Bekrefter kode i aktiv linje */
         }
         if(idlst->name[0])            /* Hvis skjekking p identifikator */
         {
            if(*line != '(')           /* '(' betyr at id er funksjonskall */
            {
               /* Lagrer element til id-liste i midlertidig fil for dette: */
               if (FWrite (idlst, sizeof (idlstTYPE), 1, idlistf) != 1)
                  return (ERROR);
               finf->cid++;            /* Teller identifikatorer i kildefil */
            }
            reset_idlsttype (idlst);
         }
      }

      if (scan->comb)                  /* Start sking etter komentarslutt */
      {
         for (; *(line + 1); line++)   /* Ske etter komentarslutt */
         {
            if(*line != ' ' &&         /* Annet tegn enn ' ' er registrert */
               *line != '\t' &&        /* Tabulator-tegn */
               *line != '\n')          /* Linjeskift */
            {
               scan->emp = FALSE;      /* Bekrefter annet tegn enn ' ' funnet*/
            }
            if(*line == '*' &&         /* Frste tegn i komentarslutt-merke */
               *(line + 1) == '/')     /* Andre tegn i komentarslutt-merke */
            {
               scan->comb = FALSE;     /* Slutt p komentarblokk */
               line++;                 /* Peker oppdateres til forbi komentar*/
               if (!scan->comb)        /* Hvis utenfor komentarblokk */
                  break;               /* Forlat komentarslutt-sk lkken */
            }
         }
         continue;                     /* Fortsett scanning av linje */
      }

      else
      if(*line == '/' &&               /* Frste tegn til komentarblokk-start*/
         *(line + 1) == '*' &&         /* Andre tegn til komentarblokk-slutt */
         !scan->quot)                  /* Kun hvis utenfor gseyneniv */
      {
         scan->comb = TRUE;            /* Komentarblokk har startet */
         finf->com++;                  /* En ny komentarblokk er registrert */
         if (func->name[0])            /* Hvis innenfor mulig funksjonshode */
            func->com++;               /* Teller komentarblokker i funksjon */
         line++;                       /* Peker n til selve komentarstarten */
         continue;                     /* Fortsett scanning av linje */
      }

      else
      if(*line == '/' &&               /* Frste tegn til komentarblokk-start*/
         *(line + 1) == '/' &&         /* Komentar kan ogs innledes med // */
         !scan->quot)                  /* Kun hvis utenfor gseyneniv */
      {
         finf->com++;                  /* En ny komentarblokk er registrert */
         if (func->fok)                /* Hvis inne i bekreftet funksjon */
            func->com++;               /* Teller komentarblokker i funksjon */
         line++;                       /* Peker n til selve komentarstarten */
         break;                        /* Avslutt tolking av linje */
      }

      else
      if(*line != ' ' &&               /* Tegn utenfor komentar */
         *line != '\t' &&              /* Tabulatortegn */
         *line != '\n')                /* Linjeskift */
      {
         scan->ocom = FALSE;           /* Bekrefter tegn utenfor komentar */
      }

      if(*line == '\'' &&              /* Mulig kommende anfrselstegnniv */
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         !scan->drop &&                /* Kun hvis ikke slikt niv fra fr */
         !scan->quot)                  /* Kun hvis utenfor gseyneniv */
      {
         scan->drop = TRUE;            /* Herved anfrselstegn niv */
      }

      else
      if(*line == '\'' &&              /* Mulig avsluttende anfrselstegnniv*/
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         scan->drop &&                 /* Kun hvis slikt niv er aktivt */
         !scan->quot)                  /* Kun hvis utenfor gseyneniv */
      {
         scan->drop = FALSE;           /* Herved avsluttet anfrselstegnniv */
      }

      else
      if(*line == '"' &&               /* Mulig gseyneniv */
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         !scan->quot &&                /* Kun hvis ikke i slikt niv fra fr */
         !scan->drop)                  /* Kun hvis utenfor anfrselstegnniv */
      {
         scan->quot = TRUE;            /* Herved aktivt gseyneniv */
      }

      else
      if(*line == '"' &&               /* Mulig avsluttende gseyneniv */
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         scan->quot &&                 /* Kun hvis slikt niv er aktivt */
         !scan->drop)                  /* Kun hvis utenfor anfrselstegnniv */
      {
         scan->quot = FALSE;           /* Herved avsluttet gseyneniv */
      }

      else
      if(*line == '{' &&               /* Start av nytt blokkniv i kildefil */
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         !scan->quot &&                /* Kun hvis utenfor gseyneniv */
         !scan->drop)                  /* Kun hvis utenfor anfrselstegnniv */
      {
         if (scan->block == 15)        /* Maks blokkniv er 15 */
         {
            DisplayCurrentLNr (finf->ltot);
            /* Too many block levels in source! */
            RETERR (21, GetFNames (sourcef));
         }
         scan->block++;                /* Ett nytt blokkniv */
         if(func->name[0] &&           /* Hvis det underskes om funksjon */
            !func->fok &&              /* Kun hvis utenfor funksjon */
            scan->block == 1)          /* Hvis blokkstart { til frste niv */
         {
            func->fok = TRUE;          /* Hver funksjon innledes med { */
            func->blin_block = finf->ltot; /* Linjenr for blokkstart i funk.*/
            finf->func++;              /* Teller funksjoner i aktiv kildefil */
         }
      }

      else
      if(*line == '}' &&               /* Slutt p blokkniv */
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         !scan->quot &&                /* Kun hvis utenfor gseyneniv */
         !scan->drop)                  /* Kun hvis utenfor anfrselstegnniv */
      {
         if (scan->block == 0)         /* If no block to terminate */
         {
            DisplayCurrentLNr (finf->ltot);
            RETERR (25, GetFNames (sourcef)); /* Illegal level of block. Probably due to a syntax error. */
         }
         scan->block--;                /* Ute av siste blokkniv */
         if(scan->block == 0 &&        /* Siste blokkslutt til funksjon */
            func->fok)                 /* Kun hvis funksjonen er bekreftet */
         {
            /* Lagrer funksjonsinformasjon til midlertidig fil for dette: */
            if (FWrite (func, sizeof (funcTYPE), 1, funregf) != 1)
               return (ERROR);
            reset_functype (func);
         }
      }

      else
      if(*line == ')' &&               /* Mulig parantesniv lavere */
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         !scan->quot &&                /* Innenfor gseyne betyr i streng */
         !scan->drop &&                /* Kun hvis utenfor anfrselstegnniv */
         scan->brack)                  /* Kun hvis innledende '(' */
      {
         scan->brack--;                /* Et parantesniv lavere */
         while(*(line + 1) == ' ' ||   /* Sk etter neste karakter etter ) */
               *(line + 1) == '\t')    /* Tabulator-karakter */
         {
            line++;                    /* Pek til neste karakter */
         }
         if(*(line + 1) == ';' &&      /* ';' betyr at kun funksjon-header */
            !scan->block &&            /* Kun hvis utenfor blokkniv */
            !scan->brack)              /* Kun hvis utenfor parantesniv */
         {
            reset_functype (func);     /* Det var ingen funksjon likevel */
         }
      }

      else
      if(*line == '(' &&               /* Parantesstart kan bety funksjon */
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         !func->name[0] &&             /* Hvis ikke allerede sjekk om funksjo*/
         !scan->brack &&               /* Funksjonshoder kun utenfor parantes*/
         !scan->drop &&                /* Kun hvis utenfor anfrselstegnniv */
         !scan->block &&               /* Kun hvis utenfor blokkniv */
         !scan->quot)                  /* Innenfor gseyne betyr i streng */
      {
         res = line--;                 /* Husk posisjon til (. Pek forran ( */
         scan->brack++;                /* Et nytt parantesniv */
         while((*line == ' ' ||        /* Bde ' ' og '\t' (tabulatortegn) */
                *line == '\t') &&      /* kan frekomme mellom f.navn og '(' */
                line > start)          /* Ikke pek utenfor "str"'s start */
         {
            line--;                    /* Sk etter slutten av funskjonsnavn */
         }
         if(!strchr(legal, *line))     /* Tegn som er ulovlig i funksjonsnavn*/
         {
            line = res;                /* Fram igjen til parantesstart */
            continue;                  /* Fortsett tolking av linje */
         }
         while(strchr (legal, *line) != NULL && /* Sk for tegn utenfor navn */
               line > start)           /* Ikke pek utenfor "str"'s start */
         {
            line--;                    /* Sk etter start av funksjonskall */
         }
         if (strchr (legal, *line) == NULL) /* Kompanser for siste "str--" */
            line++;                    /* Peker n til frste kar. i funknavn*/
         t1 = 0;                       /* Startposisjon til navn p funksjon */
         while(*line != ' ' &&         /* Karakter som betyr slutt p navn */
               *line != '\t' &&        /* Tabulator kan finnes forran ( */
               *line != '(' &&         /* Karakter som betyr slutt p navn */
               *line &&                /* Ikke utenfor slutten til "str" */
               t1 < MAXFUNCN-1)        /* Antall lovlige tegn i funksjonnavn */
         {
            func->name[t1++] = *line++;/* Delkarakter til funskjonsnavnet */
         }
         if(t1 == MAXFUNCN-1 &&        /* Hvis funksjonnavnet er for langt */
            *line != '(' &&            /* Det sjekkes ogs i lkken ovenfor */
            *line != '\t' &&
            *line != ' ')              /* '(' og ' ' => lenden var akkurat */
         {
            DisplayCurrentLNr (finf->ltot);
            RETERR (27, NULL); /* Function name is to long! Max length is ... */
         }
         func->name[t1] = '\0';        /* Markerer slutten p funksjonsnavstr*/
         func->blin = finf->ltot;      /* Lagre linjenummer til funksjonstart*/
         func->lines++;                /* Ogs funksjonshodet utgjr en linje*/
         line = res;                   /* Fram igjen til parantesstart */
      }

      else
      if(*line == '(' &&               /* Parantesstart kan bety funksjonskal*/
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         scan->block &&                /* Funksjonskall kun innenfor blokkniv*/
         !scan->drop &&                /* Kun hvis utenfor anfrselstegnniv */
         !scan->quot)                  /* Innenfor gseyner betyr streng */
      {
         res = line--;                 /* Husk posisjon til (. Pek forran ( */
         scan->brack++;                /* Et nytt parantesniv */
         while ((*line == ' ' ||       /* Bde ' ' og '\t' (tabulatortegn) */
                 *line == '\t') &&     /* kan frekomme mellom f.navn og '(' */
                 line > start)         /* Ikke pek utenfor "str"'s start */
         {
            line--;                    /* Sk etter slutten av funskjonsnavn */
         }
         if (!strchr(legal, *line))    /* Tegn som er ulovlig i funksjonsnavn*/
         {
            line = res;                /* Fram igjen til parantesstart */
            continue;                  /* Fortsett tolking av linje */
         }
         while (strchr (legal, *line) != NULL && /* Sk for tegn utenfor navn */
                line > start)          /* Ikke pek utenfor "str"'s start */
         {
            line--;                    /* Sk etter start av funksjonskall */
         }
         if (strchr (legal, *line) == NULL) /* Kompanser for siste str-- */
            line++;                    /* Peker n til frste kar. i funknavn*/
         t1 = 0;                       /* Startposisjon til navn p funksjon */
         while(*line != ' ' &&         /* Karakter som betyr slutt p navn */
               *line != '\t' &&        /* Tabulator kan finnes forran ( */
               *line != '(' &&         /* Karakter som betyr slutt p navn */
               *line &&                /* Ikke utenfor slutten til "str" */
               t1 < MAXFUNCN-1)        /* Antall lovlige tegn i funksjonnavn */
         {
            fcal.name[t1++] = *line++; /* Delkarakter til funskjonsnavnet */
         }
         if(t1 == MAXFUNCN-1 &&        /* Hvis funksjonnavnet er for langt */
            *line != '(' &&            /* Det sjekkes ogs i lkken ovenfor */
            *line != '\t' &&
            *line != ' ')              /* '(' og ' ' => lenden var akkurat */
         {
            DisplayCurrentLNr (finf->ltot);
            RETERR (27, NULL); /* Function name is to long! Max length is ... */
         }
         fcal.name[t1] = '\0';         /* Markerer slutten p funksjonsnavstr*/
         fcal.lnr = finf->ltot;        /* Lagre linjenummer til funksjonskall*/

         /* Lagrer info om funksjonskall til midlertidig global fil "funcalf"*/
         if (ifkeyword (fcal.name) == FALSE)
         {
            if (FWrite (&fcal, sizeof (fcalTYPE), 1, funcalf) != 1)
               return (ERROR);
            finf->fcal++;              /* Teller funksjonskall i kildefil */
            func->fcal++;              /* Teller funksjonskall i funcksjon */
         }
         reset_fcaltype (&fcal);
         line = res;                   /* Fram igjen til parantesstart */
      }

      else
      if(*line == '(' &&               /* Start av nytt parantesniv */
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         !scan->drop &&                /* Kun hvis utenfor anfrselstegn */
         !scan->quot)                  /* Kun hvis utenfor gseyneniv (".")*/
      {
         scan->brack++;                /* "scan.parantes" anngir parantesniv*/
      }

      else
      if(*line == '\\' &&              /* I C/C++ definerer \ et neste tegn */
         !scan->dirl)                  /* Kun hvis linje ikke er direktiv */
      {
         line += 1;                    /* Skjul neste tegn ved  hoppe over */
         continue;                     /* Fortsett tolking av linje */
      }

      else
      if(strchr (legal, *line) &&      /* Potensiel bokstav i identifikator */
         !scan->dirl &&                /* Kun hvis linje ikke er direktiv */
         !scan->drop &&                /* Kun hvis utenfor anfrselstegn */
         !scan->quot)                  /* Kun hvis utenfor gseyneniv (".")*/
      {
         if (isdigit (*line))          /* 0..9 ikke lov som frste tegn */
         {
            /* This first character of a word is 0..9, so just skip the word */
            /* Sk etter slutten til identifikator */
            while (*line && strchr (legal, *line) && !isspace (*line))
               line++;                 /* Neste karakter, for  hoppe over */
            line--;                    /* Kompanserer for siste line++ */
            continue;
         }

         idlst->lnr = finf->ltot;      /* Lagre linjenummer til identifikator*/
         t1 = 0;                       /* Offset til streng for identifikator*/
         while(*line &&                /* Sk etter slutten til identifikator*/
               t1 < MAXFUNCN-1 &&      /* Ikke utenfor strengomrde */
               (strchr (legal, *line) || /* Lovlige tegn i identifikator */
                *line == '.' ||          /* '.' skiller elementer i struktur */
               (*line == '-' &&          /* "->" skiller elemeneter i struktu*/
                *(line + 1) == '>')))    /* '>' er det andre tegnet i "->" */
         {
            if(*line == '-' &&           /* Hvis "->"... */
               *(line + 1) == '>')
            {
               idlst->name[t1++] = '.';  /* '.' erstatter "->" i buff for id */
               line += 2;                /* Peker n til frste tegn bak "->"*/
            }
            else
            {
               idlst->name[t1++] = *line++; /* Neste karakter i identifikator*/
            }
         }
         if(t1 == MAXFUNCN-1 &&        /* Hvis identifikator var for lang */
            strchr (legal, *line))     /* Kun hvis neste tegn tilhrer idnavn*/
         {
            DisplayCurrentLNr (finf->ltot);
            RETERR (28, NULL); /* Identifier name is to long! Max length is ... */
         }
         line--;                       /* Kompanserer for siste line++       */
         idlst->name[t1] = 0;          /* Avslutter streng for identifikator */
         if (ifkeyword (idlst->name))  /* Hvis identifikator er nkkelord */
            reset_idlsttype (idlst);   /* Ikke identifikator likevel */
      }
   }

   if(scan->code ||                    /* Ikke kom.linje hvis kode i linjen */
      scan->dirl ||                    /* Ikke kom.linje hvis direktiv */
      (scan->emp && !scan->comb))      /* Ikke kom.lin hvis tom utenf. komblo*/
   {
      scan->ocom = FALSE;              /* Ikke komentarlinje */
   }

   else                                /* Hele linjen var komentar */
   {
      finf->coml++;                    /* Teller kom.linjer i aktiv kildefil */
      if (func->name[0])               /* Hvis inne i funksjonshode */
         func->coml++;                 /* Teller komentarlinjer i funksjon */
   }

   if (scan->emp)                      /* Hvis hele linjen var tom */
   {
      finf->elin++;                    /* Teller tomme linjer i aktiv kildefi*/
      if (func->fok)                   /* Hvis innenfor funksjonblokk */
         func->empl++;                 /* Teller tomme linjer i funksjon */
   }

   if(scan->code)                      /* Hvis linjen ikke var tom */
   {
      finf->codl++;                    /* Teller kodelinjer i aktiv kildefil */
      if (!scan->dirl &&               /* Hvis linje ikke er direktiv */
          func->fok)                   /* Hvis inne i bekreftet funksjon */
      {
         func->codl++;                 /* Teller linjer med kode i funksjon */
      }
   }

   return (OK);
} /* interpret_line (); */


