/* ------------------------------------------------------------------------
 *
 *        File: smgtotin.c
 *     Project: Source Mapper.
 *     Created: September 21, 1992.
 * Description: Kildekode for  skrive totalinformasjon til endelig kartfil.
 *
 * Copyright (C) 2000 Leif-Erik Larsen.
 * This file is part of the Source Mapper source package.
 * Source Mapper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, in version 2 as it comes in the
 * "COPYING" file of the XWorkplace main distribution.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ------------------------------------------------------------------------ */




#include "smg.h"




#define TOTINFCOL 8   /* Kolonneavstand mellom kildefilers totalinformasjon */




int wrt_totinfo ( FILE *mapf )
/* Opprettet: Sndag 20. september 1992.
   Parameter: "mapf" Endelig kartfil.
   Retur    : E/O.
   Beskriv  : Skriver totalinformasjon til aktivt prosjekt.
              Det forutsettes at aktuell fil er pnet og klar for skriving.
*/
{
   if (!prjstat.prjon)                 /* Hvis ikke noe aktivt prosjekt */
      return (ERROR);

   /* Inkluderer totalinformasjon */
   Display (D_HEIGH, "%s", _StrMSGINCLTOTINFO);

   if (prjstat.filc == 1)              /* Hvis bare n kildefil i prosjekt */
      return (wrt_totinfo1 (mapf, 1));
   else                                /* Hvis fler enn n kildefil i prosj. */
      return (wrt_totinfo2 (mapf));
} /* wrt_totinfo (); */




int wrt_totinfo1 ( FILE *file, int fnr )
/* Opprettet: Sndag 20. september 1992.
   Parameter: "file" Filen det skal skrives til.
              "fnr"  Nummer til kildefil i prosjekt som det skal skrives
                     totalinformasjon om (1..).
   Retur    : E/O.
   Beskriv  : Skriver totalinformasjon om en eneste kildefil i aktivt
              prosjekt.
              Det forutsettes at aktuell fil er pnet og klar for skriving.
*/
{
   int dummy;
   prjfeTYPE fdata;                    /* Prosjektdata om kildefil */

   if (!GetFDataFromPrj (fnr, &fdata)) /* Les kildefilnavn */
      RETURN_ERR;

   WrtStdLinComStart (file, stdwidth, linechr2);

   /* Skriv selve totalinformasjonen: */
   if (!wrt_totinfo_ (file, &fdata.finf, GetFName (fdata.name), &dummy))
      RETURN_ERR;

   if (o_various.gencom)               /* Hvis SMG-info skal legges i koment.*/
      WrtComEndLn (file);

   FPrintF (file, "\n\n\n");           /* Avsluttende linjeskift */

   return (OK);
} /* wrt_totinfo1 (); */




int wrt_totinfo2 ( FILE *file )
/* Opprettet: Sndag 20. september 1992.
   Parameter: "file" Filen det skal skrives til.
   Retur    : E/O.
   Beskriv  : Skriver totalinformasjon om alle kildefilene i aktivt
              prosjekt. Dersom aktivt prosjekt har bare n eneste kildefil
              skal funksjonen "_wrt_totinfo1 ()" brukes istedet for denne.
              Det forutsettes at aktuell fil er pnet og klar for skriving.
*/
{
   int iTemp;
   long ltot;
   int cinf;                           /* Antall info-strenger */
   int c1;                             /* Counter 1 */
   int iLongestFName = 0;              /* Number of chars in longest filename */
   finfTYPE  finftot;                  /* For lagring av global totalinfo */
   prjfeTYPE fdata;                    /* Prosjektdata om kildefil */

   if (!WrtStdLinComStart (file, stdwidth, linechr2))
      return (ERROR);

   /* Collect total statistics information about whole project */
   reset_finftype (&finftot);          /* Nullstill global totalinfo */

   c1 = 0;
   while (c1++ < prjstat.filc)         /* Alle kildefiler i prosjekt */
   {
      if (!GetFDataFromPrj (c1, &fdata)) /* Les kildefilnavn */
         RETURN_ERR;

      /* Legg sammen info om kildefil til global totalinfo: */
      finftot.ltot += fdata.finf.ltot;
      finftot.elin += fdata.finf.elin;
      finftot.coml += fdata.finf.coml;
      finftot.codl += fdata.finf.codl;
      finftot.com  += fdata.finf.com;
      finftot.func += fdata.finf.func;
      finftot.dirl += fdata.finf.dirl;
      finftot.fcal += fdata.finf.fcal;
      finftot.cid  += fdata.finf.cid;

      /* Find the length of the longest filename */
      iTemp = strlen (GetFName (fdata.name));
      if (iTemp > iLongestFName)
         iLongestFName = iTemp;
   }

   /* Column of filenames must newer be less tham 12 chars wide */
   if (iLongestFName < 12)
      iLongestFName = 12;

   /* Skriv forklaringer til bokstavlinja over selve totalinformasjonen: */
   if (!wrt_totinfo_ (file, &finftot, NULL, &cinf))
      RETURN_ERR;

   FPrintF (file, "\n\n");

   /* Hvis det bare er kildefilnavn (og eventuelt linjenummer) som skal
      inng i totalinformasjonen, s hopp over den del av denne funksjonen
      som tegner bokstavlinja: */
   if (cinf == 'A')
      goto WRT_SOURCEFILE_REGISTER;

   /* Skriv bokstavlinja som skal vre over totalinformasjonen: */
   c1 = 'A';                          /* A = frste info-streng */
   WritFLine (file, 7, ' ');
   FPrintF (file, "%c:", c1);
   WritFLine (file, iLongestFName, ' ');

   /* Skriv resten av bokstavlinja, d.v.s. fra om med B: --->: */
   while (c1 < cinf)                   /* Antall info-strenger */
   {
      if (!FPrintF (file, "%c:", ++c1) ||
          WritFLine (file, TOTINFCOL - 2, ' ') != TOTINFCOL - 2)
      {
         return (ERROR);
      }
   }

   /* Skriv info om global linjestart i bokstavlinjen: */
   if (!FPrintF (file, "%s:\n", _StrMSGGLOBLINSTART))
      return (ERROR);

   /* Skriv strek under bokstavlinja: */
   if (WritFLineLn (file, stdwidth, linechr1) != stdwidth + 2)
      return (ERROR);

WRT_SOURCEFILE_REGISTER:

   c1 = 0;
   ltot = 0;
   while (c1++ < prjstat.filc)         /* Alle kildefiler i prosjekt */
   {
      if (!GetFDataFromPrj (c1, &fdata)) /* Les kildefilnavn */
         RETURN_ERR;

      /* Skriv selve totalinformasjonen: */
      if (!wrt_totinfo2_ (file, &fdata.finf, c1, GetFName (fdata.name), iLongestFName))
         RETURN_ERR;

      /* Skriv eventuell info om globalt linjenummer til aktiv kildefil: */
      FPrintF (file, "%c-%ld\n", GLOBLINCHR, ltot + 1);
      ltot += fdata.finf.ltot;
   }

   /* Hvis det bare er kildefilnavn (og eventuelt linjenummer) som ble
      inkludert i totalinformasjonen, s hopp over den del av denne
      funksjonen som skriver global totalinformasjon: */
   if (cinf == 'A')
      goto WRT_ENDING_LINE;

   /* Skriv strek under kildefilenes totalinfo: */
   if (WritFLineLn (file, stdwidth, linechr1) != stdwidth + 2)
      return (ERROR);

   /* Skriv global totalinformasjon: */
   if (!wrt_totinfo2_ (file, &finftot, 0, _StrMSGTOTALYSTR, iLongestFName))
      RETURN_ERR;

   FPrintF (file, "\n");

WRT_ENDING_LINE:

   /* Skriv strek under globale totalinfo m/eventuell komentarslutt: */
   WrtStdLinComEnd (file, stdwidth, linechr2);
   FPrintF (file, "\n\n\n");

   return (OK);
} /* wrt_totinfo2 (); */




int wrt_totinfo2_ ( FILE *file, finfTYPE *finf, int fnr,
                    const char *sfname, int iLongestFName )
/* Opprettet: Mandag 21. september 1992.
   Parameter: "file"   Filen det skal skrives til.
              "finf"   Aktuell filinformasjon som skal skrives.
              "fnr"    Nummer til aktiv kildefil.
              "sfname" Navn til aktuell kildefil.
              "iLongestFName" Length of longest filename in list.
   Retur    : E/O.
   Beskriv  : Dette er kjernen til "_wrt_totinfo2 ()". Det er alts denne
              funksjonen som gjr selve arbeidet med  skrive den leste
              totalinformasjon, om aktiv kildefil, til kartfilen.
              "sfname" m bare holde selve navnet til filen, alts ikke
              drev og/eller katalog. Ellers returneres error.
              Det forutsettes at aktuell fil er pnet og klar for skriving.
              Funksjonen erstatter skriving av filnummer med mellomrom
              dersom "fnr" er 0. Dette er for bruk nr funksjonen skal
              brukes til  skrive global totalinformasjon.
*/
{
   int len;                            /* Lengden til totalinfo-streng */

   /* Skriv nummer til aktiv kildefil: */
   if (fnr > 0)
   {
      len = 7 - 2 - StrLenSI (fnr);
      FPrintF (file, "%c-%d", FILECHR, fnr);
      WritFLine (file, len, ' ');
   }
   else
   {
      WritFLine (file, 7, ' ');
   }

   /* Skriv filnavnet til oppgitt kildefil: */
   len = iLongestFName + 2 - strlen (sfname);
   FPrintF (file, "%s", sfname);
   WritFLine (file, len, ' ');

   if (o_totinfo.lin_tot)
   {
      len = TOTINFCOL - StrLenSL (finf->ltot);
      if (!FPrintF (file, "%ld", finf->ltot) ||
          WritFLine (file, len, ' ') != len)
      {
         return (ERROR);
      }
   }

   if (o_totinfo.lin_emp)
   {
      len = TOTINFCOL - StrLenSL (finf->elin);
      if (!FPrintF (file, "%ld", finf->elin) ||
          WritFLine (file, len, ' ') != len)
      {
         return (ERROR);
      }
   }

   if (o_totinfo.lin_com)
   {
      len  = TOTINFCOL - StrLenSL (finf->coml);
      if (!FPrintF (file, "%ld", finf->coml) ||
          WritFLine (file, len, ' ') != len)
      {
         return (ERROR);
      }
   }

   if (o_totinfo.lin_cod)
   {
      len = TOTINFCOL - StrLenSL (finf->codl);
      if (!FPrintF (file, "%ld", finf->codl) ||
          WritFLine (file, len, ' ') != len)
      {
         return (ERROR);
      }
   }

   if (o_totinfo.func)
   {
      len = TOTINFCOL - StrLenSL (finf->func);
      if (!FPrintF (file, "%ld", finf->func) ||
          WritFLine (file, len, ' ') != len)
      {
         return (ERROR);
      }
   }

   if (o_totinfo.com)
   {
      len = TOTINFCOL - StrLenSL (finf->com);
      if (!FPrintF (file, "%ld", finf->com) ||
          WritFLine (file, len, ' ') != len)
      {
         return (ERROR);
      }
   }

   return (OK);
} /* wrt_totinfo2_ (); */




int max_totinf_strlen ( void )
/* Opprettet: Torsdag 24. september 1992.
   Parameter:
   Retur    : Lengden til den lengste av strengene som beskriver kartfilens
              totalinformasjon.
   Beskriv  :
*/
{
   int len = 0;                        /* Lengden til aktiv streng */
   int lenmax;                         /* Lengden til hitill lengste streng */

   /* Navn til kildefil: */
   lenmax = strlen (_StrMSGTOTINF_SNAME);

   /* Antall linjer totalt: */
   if (o_totinfo.lin_tot &&
       (len = strlen (_StrMSGTOTINF_LINTOT)) > lenmax)
   {
      lenmax = len;
   }

   /* Antall tomme linjer: */
   if (o_totinfo.lin_emp &&
       (len = strlen (_StrMSGTOTINF_EMPLIN)) > lenmax)
   {
      lenmax = len;
   }

   /* Antall komentarlinjer: */
   if (o_totinfo.lin_com &&
       (len = strlen (_StrMSGTOTINF_COMLIN)) > lenmax)
   {
      lenmax = len;
   }

   /* Antall linjer med kode: */
   if (o_totinfo.lin_cod &&
       (len = strlen (_StrMSGTOTINF_CODLIN)) > lenmax)
   {
      lenmax = len;
   }

   /* Antall funksjoner: */
   if (o_totinfo.func &&
       (len = strlen (_StrMSGTOTINF_FUNTOT)) > lenmax)
   {
      lenmax = len;
   }

   /* Antall funksjoner: */
   if (o_totinfo.com &&
       (len = strlen (_StrMSGTOTINF_COMENT)) > lenmax)
   {
      lenmax = len;
   }

   return (lenmax);
} /* max_totinf_strlen (); */




int wrt_totinfo_ ( FILE *file, finfTYPE *finf, const char *sfname, int *cinf )
/* Opprettet: Mandag 21. september 1992.
   Parameter: "file"   Filen det skal skrives til.
              "finf"   Aktuell filinformasjon som skal skrives.
              "sfname" Navn til aktuell kildefil, u/drev katalog.
                       Hvis denne er NULL vill vi anta at "finf" er
                       statistikk over hele prosjektet og at det
                       derfor skal skrives totalinformasjon om
                       et prosjekt bestende av flere kildefiler.
              "cinf"   Returnerer antall elementer i totalinfo-hodet.
   Retur    : E/O.
   Beskriv  : Dette er kjernen til "_wrt_totinfo1 ()". Det er alts denne
              funksjonen som gjr selve arbeidet med  skrive
              totalinformasjon om den eneste kildefil i aktivt
              prosjekt.
              Det forutsettes at aktuell fil er pnet og klar for skriving.
*/
{
   int lenmax;

   *cinf = 'A';                        /* Frste bokstavforklaring er 'A' */

   /* Hvis bruker har satt valg slik at totalinfo ikke skal best av noe
      annet enn kildefilnavn (og eventuel linjenummer), s skal ikke
      denne funksjonen skrive noen forklaringer i det hele tatt. */

   if (!sfname && /* Hvis totalinfo om prosjekt med fler kildefiler */
       !o_totinfo.lin_tot &&
       !o_totinfo.lin_emp &&
       !o_totinfo.lin_com &&
       !o_totinfo.lin_cod &&
       !o_totinfo.func &&
       !o_totinfo.com)
   {
      return (OK);
   }

   lenmax = max_totinf_strlen ();      /* Lengden til lengste totinfo-streng */

   if (sfname)
   {
      FPrintF (file, "\n%s", _StrMSGTOTINF_SNAME);
      WritFLine (file, lenmax - strlen (_StrMSGTOTINF_SNAME), '.');
      FPrintF (file, ": %s", sfname);
   }
   else
   {
      FPrintF (file, "\n%c = %s", *cinf, _StrMSGTOTINF_SNAME);
   }

   if (o_totinfo.lin_tot)
   {
      FPrintF (file, "\n");
      if (!sfname)
      {
         FPrintF (file, "%c = ", ++(*cinf));
      }
      FPrintF (file, "%s", _StrMSGTOTINF_LINTOT);
      WritFLine (file, lenmax - strlen (_StrMSGTOTINF_LINTOT), '.');
      FPrintF (file, ": %ld", finf->ltot);
   }

   if (o_totinfo.lin_emp)
   {
      FPrintF (file, "\n");
      if (!sfname)
      {
         FPrintF (file, "%c = ", ++(*cinf));
      }
      FPrintF (file, "%s", _StrMSGTOTINF_EMPLIN);
      WritFLine (file, lenmax - strlen (_StrMSGTOTINF_EMPLIN), '.');
      FPrintF (file, ": %u", finf->elin);
      if (finf->ltot > 0)              /* Unng evnt. "Divide by zero"       */
      {
         if (!FPrintF (file, " (%d%%)", ((long) finf->elin * 100) / finf->ltot))
            return (ERROR);
      }
   }

   if (o_totinfo.lin_com)
   {
      FPrintF (file, "\n");
      if (!sfname)
      {
         FPrintF (file, "%c = ", ++(*cinf));
      }
      FPrintF (file, "%s", _StrMSGTOTINF_COMLIN);
      WritFLine (file, lenmax - strlen (_StrMSGTOTINF_COMLIN), '.');
      FPrintF (file, ": %ld", finf->coml);
      if (finf->ltot > 0)              /* Unng evnt. "Divide by zero"       */
      {
         if (!FPrintF (file, " (%d%%)", ((long) finf->coml * 100) / finf->ltot))
            return (ERROR);
      }
   }

   if (o_totinfo.lin_cod)
   {
      FPrintF (file, "\n");
      if (!sfname)
      {
         FPrintF (file, "%c = ", ++(*cinf));
      }
      FPrintF (file, "%s", _StrMSGTOTINF_CODLIN);
      WritFLine (file, lenmax - strlen (_StrMSGTOTINF_CODLIN), '.');
      FPrintF (file, ": %ld", finf->codl);
      if (finf->ltot > 0)              /* Unng evnt. "Divide by zero"       */
      {
         if (!FPrintF (file, " (%d%%)", ((long) finf->codl * 100) / finf->ltot))
            return (ERROR);
      }
   }

   if (o_totinfo.func)
   {
      FPrintF (file, "\n");
      if (!sfname)
      {
         FPrintF (file, "%c = ", ++(*cinf));
      }
      FPrintF (file, "%s", _StrMSGTOTINF_FUNTOT);
      WritFLine (file, lenmax - strlen (_StrMSGTOTINF_FUNTOT), '.');
      FPrintF (file, ": %ld", finf->func);
   }

   if (o_totinfo.com)
   {
      FPrintF (file, "\n");
      if (!sfname)
      {
         FPrintF (file, "%c = ", ++(*cinf));
      }
      FPrintF (file, "%s", _StrMSGTOTINF_COMENT);
      WritFLine (file, lenmax - strlen (_StrMSGTOTINF_COMENT), '.');
      FPrintF (file, ": %ld", finf->com);
   }

   if (errno != 0)
      return (ERROR);

   return (OK);
} /* wrt_totinfo_ (); */


